@@ -26,9 +26,9 @@
 #include "common/fdset_poll.h"
 
 #define OS_FDS_CHUNKSIZE 8   /*!< Number of pollfd structs in a chunk. */
-#define OS_FDS_KEEPCHUNKS 32 /*!< Will attempt to free memory when reached. */
 
 struct fdset_t {
+	fdset_base_t _base;
 	struct pollfd *fds;
 	nfds_t nfds;
 	size_t reserved;
@@ -39,23 +39,19 @@ struct fdset_t {
 fdset_t *fdset_poll_new()
 {
 	fdset_t *set = malloc(sizeof(fdset_t));
-	if (!set) {
-		return 0;
+	if (set != NULL) {
+		memset(set, 0, sizeof(fdset_t));
 	}
 
-	/* Blank memory. */
-	memset(set, 0, sizeof(fdset_t));
 	return set;
 }
 
 int fdset_poll_destroy(fdset_t * fdset)
 {
-	if(!fdset) {
+	if(fdset == NULL) {
 		return -1;
 	}
 
-	/*! \todo No teardown required I guess. */
-
 	/* OK if NULL. */
 	free(fdset->fds);
 	free(fdset);
@@ -64,37 +60,26 @@ int fdset_poll_destroy(fdset_t * fdset)
 
 int fdset_poll_add(fdset_t *fdset, int fd, int events)
 {
-	if (!fdset || fd < 0 || events <= 0) {
+	if (fdset == NULL || fd < 0 || events <= 0) {
 		return -1;
 	}
 
 	/* Realloc needed. */
-	if (fdset->nfds == fdset->reserved) {
-		const size_t chunk = OS_FDS_CHUNKSIZE;
-		const size_t nsize = sizeof(struct pollfd) * (fdset->reserved + chunk);
-		struct pollfd *fds_n = malloc(nsize);
-		if (!fds_n) {
-			return -1;
-		}
-
-		/* Clear and copy old fdset data. */
-		memset(fds_n, 0, nsize);
-		memcpy(fds_n, fdset->fds, fdset->nfds * sizeof(struct pollfd));
-		free(fdset->fds);
-		fdset->fds = fds_n;
-		fdset->reserved += chunk;
+	if (mreserve((char **)&fdset->fds, sizeof(struct pollfd),
+	             fdset->nfds + 1, OS_FDS_CHUNKSIZE, &fdset->reserved) < 0) {
+		return -1;
 	}
 
 	/* Append. */
-	int nid = fdset->nfds++;
+	int nid = fdset->nfds++;;
 	fdset->fds[nid].fd = fd;
-	fdset->fds[nid].events = POLLIN; /*! \todo Map events to POLL events. */
+	fdset->fds[nid].events = POLLIN;
 	return 0;
 }
 
 int fdset_poll_remove(fdset_t *fdset, int fd)
 {
-	if (!fdset || fd < 0) {
+	if (fdset == NULL || fd < 0) {
 		return -1;
 	}
 
@@ -119,14 +104,16 @@ int fdset_poll_remove(fdset_t *fdset, int fd)
 	memmove(fdset->fds + pos, fdset->fds + (pos + 1), remaining);
 	--fdset->nfds;
 
-	/*! \todo Return memory if overallocated (nfds is far lower than reserved). */
-	/*! \todo Maybe >64 free chunks is excess? */
+	/* Trim excessive memory if possible (retval is not interesting). */
+	mreserve((char **)&fdset->fds, sizeof(struct pollfd), fdset->nfds,
+	         OS_FDS_CHUNKSIZE, &fdset->reserved);
+	
 	return 0;
 }
 
-int fdset_poll_wait(fdset_t *fdset)
+int fdset_poll_wait(fdset_t *fdset, int timeout)
 {
-	if (!fdset || fdset->nfds < 1 || !fdset->fds) {
+	if (fdset == NULL || fdset->nfds < 1 || fdset->fds == NULL) {
 		return -1;
 	}
 
@@ -135,7 +122,7 @@ int fdset_poll_wait(fdset_t *fdset)
 	fdset->begin = 0;
 
 	/* Poll for events. */
-	int ret = poll(fdset->fds, fdset->nfds, -1);
+	int ret = poll(fdset->fds, fdset->nfds, timeout);
 	if (ret < 0) {
 		return -1;
 	}
@@ -148,7 +135,7 @@ int fdset_poll_wait(fdset_t *fdset)
 
 int fdset_poll_begin(fdset_t *fdset, fdset_it_t *it)
 {
-	if (!fdset || !it) {
+	if (fdset == NULL || it == NULL) {
 		return -1;
 	}
 
@@ -159,7 +146,7 @@ int fdset_poll_begin(fdset_t *fdset, fdset_it_t *it)
 
 int fdset_poll_end(fdset_t *fdset, fdset_it_t *it)
 {
-	if (!fdset || !it || fdset->nfds < 1) {
+	if (fdset == NULL || it == NULL || fdset->nfds < 1) {
 		return -1;
 	}
 
@@ -188,7 +175,7 @@ int fdset_poll_end(fdset_t *fdset, fdset_it_t *it)
 
 int fdset_poll_next(fdset_t *fdset, fdset_it_t *it)
 {
-	if (!fdset || !it || fdset->nfds < 1) {
+	if (fdset == NULL || it == NULL || fdset->nfds < 1) {
 		return -1;
 	}
 
@@ -197,7 +184,7 @@ int fdset_poll_next(fdset_t *fdset, fdset_it_t *it)
 		struct pollfd* pfd = fdset->fds + it->pos;
 		if (pfd->events & pfd->revents) {
 			it->fd = pfd->fd;
-			it->events = pfd->revents; /*! \todo MAP events. */
+			it->events = pfd->revents;
 			++it->pos; /* Next will start after current. */
 			return 0;
 		}
