@@ -19,6 +19,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <getopt.h>
+#include <limits.h>
 #ifdef HAVE_CAP_NG_H
 #include <cap-ng.h>
 #endif /* HAVE_CAP_NG_H */
@@ -38,6 +39,7 @@
 /* Signal flags. */
 static volatile short sig_req_stop = 0;
 static volatile short sig_req_reload = 0;
+static volatile short sig_req_refresh = 0;
 static volatile short sig_stopping = 0;
 
 // SIGINT signal handler
@@ -48,7 +50,13 @@ void interrupt_handle(int s)
 		sig_req_reload = 1;
 		return;
 	}
-
+	
+	// Refresh
+	if (s == SIGUSR2) {
+		sig_req_refresh = 1;
+		return;
+	}
+	
 	// Stop server
 	if (s == SIGINT || s == SIGTERM) {
 		if (sig_stopping == 0) {
@@ -165,24 +173,15 @@ int main(int argc, char **argv)
 	if (config_fn[0] != '/')
 	{
 		// Get absolute path to cwd
-		size_t cwbuflen = 64;
-		char *cwbuf = malloc((cwbuflen + 2) * sizeof(char));
-		while (getcwd(cwbuf, cwbuflen) == 0) {
-			cwbuflen *= 2;
-			cwbuf = realloc(cwbuf, (cwbuflen + 2) * sizeof(char));
+		char *rpath = realpath(config_fn, NULL);
+		if (rpath == NULL) {
+			log_server_error("Couldn't get absolute path for configuration file '%s' - "
+			                 "%s.\n", config_fn, strerror(errno));
+			return 1;
+		} else {
+			free(config_fn);
+			config_fn = rpath;
 		}
-		cwbuflen = strlen(cwbuf);
-
-		// Append ending slash
-		if (cwbuf[cwbuflen - 1] != '/') {
-			cwbuf = strncat(cwbuf, "/", 1);
-		}
-
-		// Assemble path to config file
-		char *abs_cfg = strcdup(cwbuf, config_fn);
-		free(config_fn);
-		free(cwbuf);
-		config_fn = abs_cfg;
 	}
 	
 	/* POSIX 1003.1e capabilities. */
@@ -290,6 +289,7 @@ int main(int argc, char **argv)
 		sigaction(SIGTERM, &sa, NULL);
 		sigaction(SIGHUP,  &sa, NULL);
 		sigaction(SIGPIPE, &sa, NULL);
+		sigaction(SIGUSR2, &sa, NULL);
 		sa.sa_flags = 0;
 		pthread_sigmask(SIG_BLOCK, &sa.sa_mask, NULL);
 
@@ -326,6 +326,17 @@ int main(int argc, char **argv)
 					break;
 				}
 			}
+			if (sig_req_refresh) {
+				log_server_info("Refreshing slave zones...\n");
+				sig_req_reload = 0;
+				int cf_ret = server_refresh(server);
+				if (cf_ret != KNOTD_EOK) {
+					log_server_error("Couldn't refresh "
+					                 "slave zones - %s",
+					                 knotd_strerror(cf_ret));
+				}
+				
+			}
 
 			/* Events. */
 			if (ret > 0) {
