@@ -47,9 +47,11 @@
 #ifdef KNOT_COMPILER_DEBUG
   #define KNOTD_ZDUMP_DEBUG
   #define KNOTD_ZLOAD_DEBUG
+  #define KNOTD_SEMCHECK_DEBUG
+  #define KNOTD_COMPILE_DEBUG
 #endif
 
-#include "knot/other/log.h"
+#include "common/log.h"
 #include "common/print.h"
 
 /******************************************************************************/
@@ -434,6 +436,90 @@
 #define dbg_zload_exec_detail(cmds)
 #endif
 
+#ifdef KNOTD_COMPILER_DEBUG
+
+/* Brief messages. */
+#ifdef DEBUG_ENABLE_BRIEF
+#define dbg_zp(msg...) log_msg(LOG_SERVER, LOG_DEBUG, msg)
+#define dbg_zp_hex(data, len) hex_log(LOG_SERVER, (data), (len))
+#else
+#define dbg_zp(msg...)
+#define dbg_zp_hex(data, len)
+#endif
+
+/* Verbose messages. */
+#ifdef DEBUG_ENABLE_VERBOSE
+#define dbg_zp_verb(msg...) log_msg(LOG_SERVER, LOG_DEBUG, msg)
+#define dbg_zp_hex_verb(data, len) hex_log(LOG_SERVER, (data), (len))
+#else
+#define dbg_zp_verb(msg...)
+#define dbg_zp_hex_verb(data, len)
+#endif
+
+/* Detail messages. */
+#ifdef DEBUG_ENABLE_DETAILS
+#define dbg_zp_detail(msg...) log_msg(LOG_SERVER, LOG_DEBUG, msg)
+#define dbg_zp_hex_detail(data, len) hex_log(LOG_SERVER, (data), (len))
+#define dbg_zp_exec_detail(cmds) do { cmds } while (0)
+#else
+#define dbg_zp_detail(msg...)
+#define dbg_zp_hex_detail(data, len)
+#define dbg_zp_exec_detail(cmds)
+#endif
+
+/* No messages. */
+#else
+#define dbg_zp(msg...)
+#define dbg_zp_hex(data, len)
+#define dbg_zp_verb(msg...)
+#define dbg_zp_hex_verb(data, len)
+#define dbg_zp_detail(msg...)
+#define dbg_zp_hex_detail(data, len)
+#define dbg_zp_exec_detail(cmds)
+#endif
+
+#ifdef KNOTD_SEMCHECK_DEBUG
+
+/* Brief messages. */
+#ifdef DEBUG_ENABLE_BRIEF
+#define dbg_semcheck(msg...) log_msg(LOG_SERVER, LOG_DEBUG, msg)
+#define dbg_semcheck_hex(data, len) hex_log(LOG_SERVER, (data), (len))
+#else
+#define dbg_semcheck(msg...)
+#define dbg_semcheck_hex(data, len)
+#endif
+
+/* Verbose messages. */
+#ifdef DEBUG_ENABLE_VERBOSE
+#define dbg_semcheck_verb(msg...) log_msg(LOG_SERVER, LOG_DEBUG, msg)
+#define dbg_semcheck_hex_verb(data, len) hex_log(LOG_SERVER, (data), (len))
+#else
+#define dbg_semcheck_verb(msg...)
+#define dbg_semcheck_hex_verb(data, len)
+#endif
+
+/* Detail messages. */
+#ifdef DEBUG_ENABLE_DETAILS
+#define dbg_semcheck_detail(msg...) log_msg(LOG_SERVER, LOG_DEBUG, msg)
+#define dbg_semcheck_hex_detail(data, len) hex_log(LOG_SERVER, (data), (len))
+#define dbg_semcheck_exec_detail(cmds) do { cmds } while (0)
+#else
+#define dbg_semcheck_detail(msg...)
+#define dbg_semcheck_hex_detail(data, len)
+#define dbg_semcheck_exec_detail(cmds)
+#endif
+
+/* No messages. */
+#else
+#define dbg_semcheck(msg...)
+#define dbg_semcheck_hex(data, len)
+#define dbg_semcheck_verb(msg...)
+#define dbg_semcheck_hex_verb(data, len)
+#define dbg_semcheck_detail(msg...)
+#define dbg_semcheck_hex_detail(data, len)
+#define dbg_semcheck_exec_detail(cmds)
+#endif
+
 /******************************************************************************/
 
 #endif /* _KNOTD_DEBUG_H_ */
