@@ -21,10 +21,13 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
+#ifdef HAVE_CAP_NG_H
+#include <cap-ng.h>
+#endif /* HAVE_CAP_NG_H */
 
 #include "knot/common.h"
 #include "knot/server/dthreads.h"
-#include "knot/other/log.h"
+#include "common/log.h"
 #include "knot/other/error.h"
 
 /*! \brief Lock thread state for R/W. */
@@ -120,9 +123,21 @@ static void *thread_ep(void *data)
 	sigaddset(&ignset, SIGINT);
 	sigaddset(&ignset, SIGTERM);
 	sigaddset(&ignset, SIGHUP);
-	pthread_sigmask(SIG_BLOCK, &ignset, 0); /*! \todo Review under BSD. */
+	sigaddset(&ignset, SIGPIPE);
+	sigaddset(&ignset, SIGUSR1);
+	pthread_sigmask(SIG_BLOCK, &ignset, 0); /*! \todo Review under BSD (issue #1441). */
 
 	dbg_dt("dthreads: [%p] entered ep\n", thread);
+	
+	/* Drop capabilities except FS access. */
+#ifdef HAVE_CAP_NG_H
+	if (capng_have_capability(CAPNG_EFFECTIVE, CAP_SETPCAP)) {
+		capng_type_t tp = CAPNG_EFFECTIVE|CAPNG_PERMITTED;
+		capng_clear(CAPNG_SELECT_BOTH);
+		capng_update(CAPNG_ADD, tp, CAP_DAC_OVERRIDE);
+		capng_apply(CAPNG_SELECT_BOTH);
+	}
+#endif /* HAVE_CAP_NG_H */
 
 	// Run loop
 	for (;;) {
@@ -176,7 +191,6 @@ static void *thread_ep(void *data)
 
 			// Wait for notification from unit
 			dbg_dt("dthreads: [%p] going idle\n", thread);
-			/*! \todo Check return value. */
 			pthread_cond_wait(&unit->_notify, &unit->_notify_mx);
 			pthread_mutex_unlock(&unit->_notify_mx);
 			dbg_dt("dthreads: [%p] resumed from idle\n", thread);
@@ -223,8 +237,9 @@ static dthread_t *dt_create_thread(dt_unit_t *unit)
 	// Initialize attribute
 	pthread_attr_t *attr = &thread->_attr;
 	pthread_attr_init(attr);
-	pthread_attr_setinheritsched(attr, PTHREAD_INHERIT_SCHED);
-	pthread_attr_setschedpolicy(attr, SCHED_OTHER);
+	//pthread_attr_setinheritsched(attr, PTHREAD_INHERIT_SCHED);
+	//pthread_attr_setschedpolicy(attr, SCHED_OTHER);
+	pthread_attr_setstacksize(attr, 1024*1024);
 	return thread;
 }
 
@@ -801,41 +816,63 @@ int dt_stop(dt_unit_t *unit)
 	return KNOTD_EOK;
 }
 
-int dt_setprio(dthread_t *thread, int prio)
+//int dt_setprio(dthread_t *thread, int prio)
+//{
+//	// Check input
+//	if (thread == 0) {
+//		return KNOTD_EINVAL;
+//	}
+
+//	// Clamp priority
+//	int policy = SCHED_FIFO;
+//	prio = MIN(MAX(sched_get_priority_min(policy), prio),
+//		   sched_get_priority_max(policy));
+
+//	// Update scheduler policy
+//	int ret = pthread_attr_setschedpolicy(&thread->_attr, policy);
+
+//	// Update priority
+//	if (ret == 0) {
+//		struct sched_param sp;
+//		sp.sched_priority = prio;
+//		ret = pthread_attr_setschedparam(&thread->_attr, &sp);
+//	}
+
+//	/* Map error codes. */
+//	if (ret != 0) {
+//		dbg_dt("dthreads: [%p] %s(%d): failed",
+//		       thread, __func__, prio);
+
+//		/* Map "not supported". */
+//		if (errno == ENOTSUP) {
+//			return KNOTD_ENOTSUP;
+//		}
+
+//		return KNOTD_EINVAL;
+//	}
+
+//	return KNOTD_EOK;
+//}
+
+int dt_setaffinity(dthread_t *thread, void *mask, size_t len)
 {
-	// Check input
-	if (thread == 0) {
+	if (thread == NULL || mask == NULL) {
 		return KNOTD_EINVAL;
 	}
-
-	// Clamp priority
-	int policy = SCHED_FIFO;
-	prio = MIN(MAX(sched_get_priority_min(policy), prio),
-		   sched_get_priority_max(policy));
-
-	// Update scheduler policy
-	int ret = pthread_attr_setschedpolicy(&thread->_attr, policy);
-
-	// Update priority
-	if (ret == 0) {
-		struct sched_param sp;
-		sp.sched_priority = prio;
-		ret = pthread_attr_setschedparam(&thread->_attr, &sp);
-	}
-
-	/* Map error codes. */
-	if (ret != 0) {
-		dbg_dt("dthreads: [%p] %s(%d): failed",
-		       thread, __func__, prio);
-
-		/* Map "not supported". */
-		if (errno == ENOTSUP) {
-			return KNOTD_ENOTSUP;
-		}
-
+	
+#ifdef HAVE_PTHREAD_SETAFFINITY_NP
+	if (len != sizeof(cpu_set_t)) {
 		return KNOTD_EINVAL;
 	}
-
+	pthread_t tid = pthread_self();
+	int ret = pthread_setaffinity_np(tid, len, (cpu_set_t*)mask);
+	if (ret < 0) {
+		return KNOTD_ERROR;
+	}
+#else
+	return KNOTD_ENOTSUP;
+#endif
+	
 	return KNOTD_EOK;
 }
 
@@ -948,20 +985,28 @@ int dt_compact(dt_unit_t *unit)
 	return KNOTD_EOK;
 }
 
+int dt_online_cpus()
+{
+	int ret = -1;
+#ifdef _SC_NPROCESSORS_ONLN
+	ret = (int) sysconf(_SC_NPROCESSORS_ONLN);
+#endif
+	return ret;
+}
+
 int dt_optimal_size()
 {
-#ifdef _SC_NPROCESSORS_ONLN
-	int ret = (int) sysconf(_SC_NPROCESSORS_ONLN);
-	if (ret >= 1) {
+	int ret = dt_online_cpus();
+	if (ret > 0) {
 		return ret + CPU_ESTIMATE_MAGIC;
 	}
-#endif
+	
 	dbg_dt("dthreads: failed to fetch the number of online CPUs.");
 	return DEFAULT_THR_COUNT;
 }
 
 /*!
- * \todo Use memory barriers or asynchronous read-only access, locking
+ * \note Use memory barriers or asynchronous read-only access, locking
  *       poses a thread performance decrease by 1.31%.
  */
 
@@ -978,6 +1023,22 @@ int dt_is_cancelled(dthread_t *thread)
 	return ret;
 }
 
+unsigned dt_get_id(dthread_t *thread)
+{
+	if (thread == NULL || thread->unit == NULL) {
+		return 0;
+	}
+
+	dt_unit_t *unit = thread->unit;
+	for(unsigned tid = 0; tid < unit->size; ++tid) {
+		if (thread == unit->threads[tid]) {
+			return tid;
+		}
+	}
+	
+	return 0;
+}
+
 int dt_unit_lock(dt_unit_t *unit)
 {
 	// Check input
