@@ -165,8 +165,6 @@ int server_remove_handler(server_t *server, iohandler_t *ref);
  * \retval KNOTD_EOK on success.
  * \retval KNOTD_EINVAL on invalid parameters.
  *
- * \todo When a module for configuration is added, the filename parameter will
- *       be removed.
  */
 int server_start(server_t *server);
 
@@ -180,6 +178,16 @@ int server_start(server_t *server);
  */
 int server_wait(server_t *server);
 
+/*!
+ * \brief Refresh served zones.
+ *
+ * \param server Server structure to be used for operation.
+ *
+ * \retval  0 On success (EOK).
+ * \retval <0 If an error occured (EINVAL).
+ */
+int server_refresh(server_t *server);
+
 /*!
  * \brief Requests server to stop.
  *
