@@ -53,7 +53,7 @@ typedef struct tcp_worker_t {
 /*
  * Forward decls.
  */
-#define TCP_THROTTLE_LO 10 /*!< Minimum recovery time on errors. */
+#define TCP_THROTTLE_LO 5 /*!< Minimum recovery time on errors. */
 #define TCP_THROTTLE_HI 50 /*!< Maximum recovery time on errors. */
 
 /*! \brief Calculate TCP throttle time (random). */
@@ -119,7 +119,7 @@ static void tcp_sweep(fdset_t *set, int fd, void* data)
 #endif
 	}
 	
-	log_server_notice("Connection with %s:%d was terminated due to "
+	log_server_notice("Connection with '%s@%d' was terminated due to "
 	                  "inactivity.\n", r_addr, r_port);
 	fdset_remove(set, fd);
 	close(fd);
@@ -132,6 +132,13 @@ static void tcp_sweep(fdset_t *set, int fd, void* data)
  *
  * \param w Associated I/O event.
  * \param revents Returned events.
+ *
+ * \note We do not know if the packet makes sense or if it is
+ *       a bunch of random bytes. There is no way to find out
+ *       without parsing. However, it is irrelevant if we copy
+ *       these random bytes to the response, so we may do it
+ *       and ensure that in case of good packet the response
+ *       is proper.
  */
 static int tcp_handle(tcp_worker_t *w, int fd, uint8_t *qbuf, size_t qbuf_maxlen)
 {
@@ -164,34 +171,39 @@ static int tcp_handle(tcp_worker_t *w, int fd, uint8_t *qbuf, size_t qbuf_maxlen
 			char r_addr[SOCKADDR_STRLEN];
 			sockaddr_tostr(&addr, r_addr, sizeof(r_addr));
 			int r_port = sockaddr_portnum(&addr);
-			log_server_warning("Couldn't receive query from %s:%d "
-			                  "within the time limit %ds.\n",
-			                  r_addr, r_port, TCP_ACTIVITY_WD);
+			log_server_warning("Couldn't receive query from '%s@%d'"
+			                  " within the time limit of %ds.\n",
+			                   r_addr, r_port, TCP_ACTIVITY_WD);
 		}
 		return KNOTD_ECONNREFUSED;
 	}
 
 	/* Parse query. */
-//	knot_response_t *resp = knot_response_new(qbuf_maxlen);
 	size_t resp_len = qbuf_maxlen; // 64K
 	knot_packet_type_t qtype = KNOT_QUERY_NORMAL;
 	knot_packet_t *packet =
 		knot_packet_new(KNOT_PACKET_PREALLOC_QUERY);
 	if (packet == NULL) {
-		uint16_t pkt_id = knot_wire_get_id(qbuf);
-		knot_ns_error_response(ns, pkt_id, KNOT_RCODE_SERVFAIL,
-				       qbuf, &resp_len);
-		tcp_reply(fd, qbuf, resp_len);
+		int ret = knot_ns_error_response_from_query(ns, qbuf, n,
+		                                            KNOT_RCODE_SERVFAIL,
+		                                            qbuf, &resp_len);
+
+		if (ret == KNOT_EOK) {
+			tcp_reply(fd, qbuf, resp_len);
+		}
+
 		return KNOTD_EOK;
 	}
 
 	int parse_res = knot_ns_parse_packet(qbuf, n, packet, &qtype);
 	if (unlikely(parse_res != KNOT_EOK)) {
 		if (parse_res > 0) { /* Returned RCODE */
-			uint16_t pkt_id = knot_wire_get_id(qbuf);
-			knot_ns_error_response(ns, pkt_id, parse_res,
-					       qbuf, &resp_len);
-			tcp_reply(fd, qbuf, resp_len);
+			int ret = knot_ns_error_response_from_query(ns, qbuf, n,
+					parse_res, qbuf, &resp_len);
+
+			if (ret == KNOT_EOK) {
+				tcp_reply(fd, qbuf, resp_len);
+			}
 		}
 		knot_packet_free(&packet);
 		return KNOTD_EOK;
@@ -207,7 +219,8 @@ static int tcp_handle(tcp_worker_t *w, int fd, uint8_t *qbuf, size_t qbuf_maxlen
 	case KNOT_QUERY_NORMAL:
 		//res = knot_ns_answer_normal(ns, packet, qbuf, &resp_len);
 		if (zones_normal_query_answer(ns, packet, &addr,
-		                              qbuf, &resp_len) == KNOT_EOK) {
+		                              qbuf, &resp_len,
+		                              NS_TRANSPORT_TCP) == KNOT_EOK) {
 			res = KNOTD_EOK;
 		}
 		break;
@@ -223,6 +236,7 @@ static int tcp_handle(tcp_worker_t *w, int fd, uint8_t *qbuf, size_t qbuf_maxlen
 		res = xfr_request_init(&xfr, xfrt, XFR_FLAG_TCP, packet);
 		if (res != KNOTD_EOK) {
 			knot_ns_error_response(ns, knot_packet_id(packet),
+			                       &packet->header.flags1,
 			                       KNOT_RCODE_SERVFAIL, qbuf,
 			                       &resp_len);
 			res = KNOTD_EOK;
@@ -239,8 +253,8 @@ static int tcp_handle(tcp_worker_t *w, int fd, uint8_t *qbuf, size_t qbuf_maxlen
 		
 	case KNOT_QUERY_UPDATE:
 		knot_ns_error_response(ns, knot_packet_id(packet),
-				       KNOT_RCODE_NOTIMPL, qbuf,
-				       &resp_len);
+		                       &packet->header.flags1,
+		                       KNOT_RCODE_NOTIMPL, qbuf, &resp_len);
 		res = KNOTD_EOK;
 		break;
 		
@@ -255,16 +269,16 @@ static int tcp_handle(tcp_worker_t *w, int fd, uint8_t *qbuf, size_t qbuf_maxlen
 	case KNOT_RESPONSE_AXFR:   /*!< Processed in XFR handler. */
 	case KNOT_RESPONSE_IXFR:   /*!< Processed in XFR handler. */
 		knot_ns_error_response(ns, knot_packet_id(packet),
-				       KNOT_RCODE_REFUSED, qbuf,
-				       &resp_len);
+		                       &packet->header.flags1,
+		                       KNOT_RCODE_REFUSED, qbuf, &resp_len);
 		res = KNOTD_EOK;
 		break;
 		
 	/* Unknown opcodes. */
 	default:
 		knot_ns_error_response(ns, knot_packet_id(packet),
-				       KNOT_RCODE_FORMERR, qbuf,
-				       &resp_len);
+		                       &packet->header.flags1,
+		                       KNOT_RCODE_FORMERR, qbuf, &resp_len);
 		res = KNOTD_EOK;
 		break;
 	}
@@ -290,7 +304,6 @@ static int tcp_accept(int fd)
 	/* Evaluate connection. */
 	if (incoming < 0) {
 		int en = errno;
-		/*! \todo Better solution so it doesn't block current connections (issue #1542). */
 		if (en != EINTR) {
 			log_server_error("Cannot accept connection "
 					 "(%d).\n", errno);
