@@ -34,12 +34,12 @@
  * \brief Dumps given zone to text (BIND-like) file.
  *
  * \param zone Zone to be saved.
- * \param fd File descriptor to write to.
+ * \param File file to write to.
  *
  * \retval KNOT_EOK on success.
  * \retval KNOT_EBADARG if the specified file is not valid for writing.
  */
-int zone_dump_text(knot_zone_contents_t *zone, int fd);
+int zone_dump_text(knot_zone_contents_t *zone, FILE *f);
 
 #endif // _KNOT_ZONE_DUMP_TEXT_H_
 
