@@ -40,26 +40,14 @@
  * \param y Second time_t value to be compared.
  *
  * \retval 0 when times are the some.
- * \retval 1 when y < x.
- * \retval -1 when x > y.
+ * \retval 1 when x > y.
+ * \retval -1 when x < y.
  */
 static int timet_cmp(time_t x, time_t y)
 {
-	/* Calculate difference in the scale of seconds. */
-	long diff = x - y;
-
-	/* X and Y are equal. */
-	if (diff == 0) {
-		return 0;
-	}
-
-	/* X is newer. */
-	if (diff > 0) {
-		return 1;
-	}
-
-	/* Y is newer. */
-	return -1;
+	if (x > y) return 1;
+	if (x < y) return -1;
+	return 0;
 }
 
 /*!
@@ -160,6 +148,9 @@ static void load_rdata_purge(knot_rdata_t *rdata,
 
 static knot_dname_t *read_dname_with_id(FILE *f)
 {
+	if (f == NULL) {
+		dbg_zload("zload: read_dname_id: NULL file.\n");
+	}
 	knot_dname_t *ret = knot_dname_new();
 	CHECK_ALLOC_LOG(ret, NULL);
 
@@ -167,16 +158,18 @@ static knot_dname_t *read_dname_with_id(FILE *f)
 	uint32_t dname_id = 0;
 	if (!fread_wrapper(&dname_id, sizeof(dname_id), 1, f)) {
 		knot_dname_release(ret);
+		dbg_zload("zload: read_dname_id: Cannot read dname ID.\n");
 		return NULL;
 	}
 
 	ret->id = dname_id;
-	dbg_zload("loaded: dname id: %u\n", dname_id);
+	dbg_zload_detail("zload: read_dname_id: dname id: %u\n", dname_id);
 
 	/* Read size of dname. */
 	uint32_t dname_size = 0;
 	if (!fread_wrapper(&dname_size, sizeof(dname_size), 1, f)) {
 		knot_dname_release(ret);
+		dbg_zload("zload: read_dname_id: Cannot read dname size.\n");
 		return NULL;
 	}
 	ret->size = dname_size;
@@ -194,6 +187,7 @@ static knot_dname_t *read_dname_with_id(FILE *f)
 
 	if (!fread_wrapper(ret->name, sizeof(uint8_t), ret->size, f)) {
 		knot_dname_release(ret);
+		dbg_zload("zload: read_dname_id: Cannot read dname name.\n");
 		return NULL;
 	}
 
@@ -201,6 +195,8 @@ static knot_dname_t *read_dname_with_id(FILE *f)
 	uint16_t label_count = 0;
 	if (!fread_wrapper(&label_count, sizeof(label_count), 1, f)) {
 		knot_dname_release(ret);
+		dbg_zload("zload: read_dname_id: Cannot read "
+		          "dname label count.\n");
 		return NULL;
 	}
 
@@ -216,11 +212,16 @@ static knot_dname_t *read_dname_with_id(FILE *f)
 	if (!fread_wrapper(ret->labels, sizeof(uint8_t), ret->label_count, f)) {
 		free(ret->name);
 		free(ret);
+		dbg_zload("zload: read_dname_id: Cannot read dname labels.\n");
 		return NULL;
 	}
 
-	dbg_zload("loaded: %s (id: %d)\n", knot_dname_to_str(ret),
+dbg_zload_exec_detail(
+	char *name = knot_dname_to_str(ret);
+	dbg_zload_detail("zload: Loaded dname: %s (id: %d).\n", name,
 			 ret->id);
+	free(name);
+);
 
 	return ret;
 }
@@ -239,6 +240,7 @@ static knot_rdata_t *knot_load_rdata(uint16_t type, FILE *f,
 {
 	knot_rdata_t *rdata = knot_rdata_new();
 	if (rdata == NULL) {
+		dbg_zload("zload: load_rdata: Cannot create new rdata.\n");
 		return NULL;
 	}
 
@@ -246,28 +248,34 @@ static knot_rdata_t *knot_load_rdata(uint16_t type, FILE *f,
 		knot_rrtype_descriptor_by_type(type);
 	assert(desc != NULL);
 
-
-	/* First, should read rdata count. */
+	/* First we should read rdata count. */
 
 	uint32_t rdata_count = 0;
 
 	if(!fread_wrapper(&rdata_count, sizeof(rdata_count), 1, f)) {
 		knot_rdata_free(&rdata);
+		dbg_zload("zload: load_rdata: Cannot read rdata count.\n");
 		return NULL;
 	}
 
 	knot_rdata_item_t *items =
 		malloc(sizeof(knot_rdata_item_t) * rdata_count);
-
-	if (desc->fixed_items) {
-		assert(desc->length == rdata_count);
+	if (items == NULL) {
+		ERR_ALLOC_FAILED;
+		free(items);
+		return NULL;
 	}
 
-	uint16_t raw_data_length;
+	if (rdata_count > desc->length) {
+		dbg_zload("zload: load_rdata: Read wrong count of RDATA.\n");
+		return NULL;
+	}
 
-	dbg_zload("Reading %d items\n", rdata_count);
+	uint16_t raw_data_length = 0;
 
-	dbg_zload("current type: %s\n", knot_rrtype_to_string(type));
+	dbg_zload_detail("zload: load_rdata: Reading %d items\n", rdata_count);
+	dbg_zload_detail("zload: load_rdata: Current type: %s\n",
+	                  knot_rrtype_to_string(type));
 
 	for (int i = 0; i < rdata_count; i++) {
 		if (desc->wireformat[i] == KNOT_RDATA_WF_COMPRESSED_DNAME ||
@@ -283,8 +291,12 @@ static knot_rdata_t *knot_load_rdata(uint16_t type, FILE *f,
 			uint8_t in_the_zone = 0;
 
 			if (use_ids) {
-				if(!fread_wrapper(&dname_id, sizeof(dname_id), 1, f)) {
-					load_rdata_purge(rdata, items, i, desc, type);
+				if(!fread_wrapper(&dname_id, sizeof(dname_id),
+				                  1, f)) {
+					load_rdata_purge(rdata,
+					                 items, i, desc, type);
+					dbg_zload("zload: load_rdata: "
+					          "Cannot read dname ID.\n");
 					return NULL;
 				}
 
@@ -298,12 +310,16 @@ static knot_rdata_t *knot_load_rdata(uint16_t type, FILE *f,
 			if(!fread_wrapper(&in_the_zone, sizeof(in_the_zone),
 			               1, f)) {
 				load_rdata_purge(rdata, items, i, desc, type);
+				dbg_zload("zload: load_rdata: "
+				          "Cannot read zone bit.\n");
 				return NULL;
 			}
 
 			if(!fread_wrapper(&has_wildcard, sizeof(uint8_t),
 				       1, f)) {
 				load_rdata_purge(rdata, items, i, desc, type);
+				dbg_zload("zload: load_rdata: "
+				          "Cannot read wildcard bit.\n");
 				return NULL;
 			}
 
@@ -312,11 +328,14 @@ static knot_rdata_t *knot_load_rdata(uint16_t type, FILE *f,
 					       1, f)) {
 					load_rdata_purge(rdata, items,
 							 i, desc, type);
+					dbg_zload("zload: load_rdata: "
+					          "Cannot read wc ID.\n");
 					return NULL;
 				}
 				items[i].dname->node =
-						id_array[dname_id]->node;
-			} else if (use_ids && !in_the_zone) { /* destroy the node */
+					id_array[dname_id]->node;
+			} else if (use_ids && !in_the_zone) {
+				/* destroy the node */
 				if (id_array[dname_id]->node != NULL) {
 					knot_node_free(&id_array[dname_id]->
 							 node, 0);
@@ -328,37 +347,51 @@ static knot_rdata_t *knot_load_rdata(uint16_t type, FILE *f,
 			if (!fread_wrapper(&raw_data_length,
 					sizeof(raw_data_length), 1, f)) {
 				load_rdata_purge(rdata, items, i, desc, type);
+				dbg_zload("zload: load_rdata: Cannot read "
+				          "raw data length.\n");
 				return NULL;
 			}
 			
 			/*!< \todo this is not proper fix, see #1678 */
-			items[i].raw_data = (uint16_t *)
-				malloc(sizeof(uint8_t) * (raw_data_length + 2));
-			items[i].raw_data[0] = raw_data_length;
-
-			if (!fread_wrapper(items[i].raw_data + 1, sizeof(uint8_t),
-			      raw_data_length, f)) {
-				load_rdata_purge(rdata, items, i + 1, desc, type);
+			items[i].raw_data =
+				malloc(raw_data_length + 2);
+			if (items[i].raw_data == NULL) {
+				ERR_ALLOC_FAILED;
+				load_rdata_purge(rdata, items, i + 1, desc,
+				                 type);
 				return NULL;
 			}
-			dbg_zload("read raw_data len %d\n", raw_data_length);
+			items[i].raw_data[0] = raw_data_length;
+
+			if (!fread_wrapper(items[i].raw_data + 1,
+			                   sizeof(uint8_t),
+			      raw_data_length, f)) {
+				load_rdata_purge(rdata, items, i + 1, desc,
+				                 type);
+				dbg_zload("zload: load_rdata: Cannot read "
+				          "raw data.\n");
+				return NULL;
+			}
+			dbg_zload_detail("zload: load_rdata: Read raw_data "
+			                 "length=%d.\n",
+			                 raw_data_length);
 		}
 	}
 
 	/* Each item has refcount already incremented for saving in rdata. */
 	if (knot_rdata_set_items(rdata, items, rdata_count) != 0) {
-		fprintf(stderr, "zoneload: Could not set items "
+		fprintf(stderr, "zload: read_rdata: Could not set items "
 			"when loading rdata.\n");
+		knot_rdata_deep_free(&rdata, type, 0);
+		return NULL;
 	}
 
 	free(items);
-
-	dbg_zload("knot_load_rdata: all %d items read\n",
-			 desc->length);
-
 	assert(rdata->count == rdata_count);
-
-	rdata->count = rdata_count;
+	
+	dbg_zload_detail("zload: read_rdata: All %d items read "
+	                 "successfully.\n",
+	                 desc->length);
 
 	return rdata;
 }
@@ -373,54 +406,77 @@ static knot_rdata_t *knot_load_rdata(uint16_t type, FILE *f,
 static knot_rrset_t *knot_load_rrsig(FILE *f, knot_dname_t **id_array,
                                          int use_ids)
 {
-	knot_rrset_t *rrsig;
+	if (f == NULL || id_array == NULL) {
+		dbg_zload("zload: load_rrsig: Bad arguments.\n");
+		return NULL;
+	}
+	
+	knot_rrset_t *rrsig = NULL;
 
-	uint16_t rrset_type;
-	uint16_t rrset_class;
-	uint32_t rrset_ttl;
+	uint16_t rrset_type = 0;
+	uint16_t rrset_class = 0;
+	uint32_t rrset_ttl = 0;
 
-	uint32_t rdata_count;
+	uint32_t rdata_count = 0;
 
 	if (!fread_wrapper(&rrset_type, sizeof(rrset_type), 1, f)) {
+		dbg_zload("zload: load_rrsig: Cannot read type.\n");
 		return NULL;
 	}
 
 	if (rrset_type != KNOT_RRTYPE_RRSIG) {
-		fprintf(stderr, "!! Error: rrsig has wrong type\n");
+		dbg_zload("zload: load_rrsig: RRSIG has wrong type,"
+		          " probably data corruption.\n");
 		return NULL;
 	}
-	dbg_zload("rrset type: %d\n", rrset_type);
+	dbg_zload_detail("zload: load_rrsig: RRSIG type: %d\n", rrset_type);
 	if (!fread_wrapper(&rrset_class, sizeof(rrset_class), 1, f)) {
+		dbg_zload("zload: load_rrsig: Cannot read class.\n");
 		return NULL;
 	}
-	dbg_zload("rrset class %d\n", rrset_class);
+	dbg_zload_detail("zload: load_rrsig: Class=%d\n", rrset_class);
 
 	if (!fread_wrapper(&rrset_ttl, sizeof(rrset_ttl), 1, f)) {
+		dbg_zload("zload: load_rrsig: Cannot read TTL.\n");
 		return NULL;
 	}
-	dbg_zload("rrset ttl %d\n", rrset_ttl);
+	dbg_zload_detail("zload: load_rrsig: TTL=%d\n", rrset_ttl);
 
 	if (!fread_wrapper(&rdata_count, sizeof(rdata_count), 1, f)) {
+		dbg_zload("zload: load_rrsig: Cannot read rdata count.\n");
 		return NULL;
 	}
 
 	rrsig = knot_rrset_new(NULL, rrset_type, rrset_class, rrset_ttl);
+	if (rrsig == NULL) {
+		dbg_zload("zload: load_rrsig: Cannot create new RRSIG.\n");
+		return NULL;
+	}
 
-	knot_rdata_t *tmp_rdata;
+	knot_rdata_t *tmp_rdata = NULL;
 
-	dbg_zload("loading %d rdata entries\n", rdata_count);
+	dbg_zload_detail("zload: load_rrsig: Loading %d rdata entries.\n",
+	                 rdata_count);
 
 	for (int i = 0; i < rdata_count; i++) {
 		tmp_rdata = knot_load_rdata(KNOT_RRTYPE_RRSIG, f,
 					      id_array, use_ids);
 		if (tmp_rdata) {
-			knot_rrset_add_rdata(rrsig, tmp_rdata);
+			int ret = knot_rrset_add_rdata(rrsig, tmp_rdata);
+			if (ret != KNOT_EOK) {
+				dbg_zload("zload: load_rrsig: Cannot "
+				          "add RDATA\n.");
+				knot_rrset_deep_free(&rrsig, 0, 1, 1);
+				return NULL;
+			}
 		} else {
+			dbg_zload("zload: load_rrsig: Cannot load rdata.\n");
 			knot_rrset_deep_free(&rrsig, 0, 1, 1);
 			return NULL;
 		}
 	}
 
+	dbg_zload_detail("zload: load_rrsig: RRSIG loaded successfully.\n");
 	return rrsig;
 }
 
@@ -446,55 +502,63 @@ static knot_rrset_t *knot_load_rrset(FILE *f, knot_dname_t **id_array,
 	knot_dname_t *owner = NULL;
 
 	if (!use_ids) {
-		dbg_zload("Loading owner of new RRSet from wire.\n");
+		dbg_zload_detail("zload: load_rrset: "
+		                 "Loading owner of new RRSet from wire.\n");
 		owner = read_dname_with_id(f);
+		if (owner == NULL) {
+			dbg_zload("zload: load_rrset: Cannot load owner.\n");
+			return NULL;
+		}
 	}
 
 	if (!fread_wrapper(&rrset_type, sizeof(rrset_type), 1, f)) {
+		dbg_zload("zload: load_rrset: Cannot load RRSet type.\n");
 		if (!use_ids) {
 			knot_dname_free(&owner);
 		}
 		return NULL;
 	}
-	dbg_zload("Zone load: rrset load: type: %u\n", rrset_type);
+	
+	dbg_zload_detail("zload: load_rrset: Type=%u\n", rrset_type);
 	if (!fread_wrapper(&rrset_class, sizeof(rrset_class), 1, f)) {
+		dbg_zload("zload: load_rrset: Cannot load RRSet class.\n");
 		if (!use_ids) {
 			knot_dname_free(&owner);
 		}
 		return NULL;
 	}
-	dbg_zload("Zone load: rrset class: type: %u\n", rrset_class);
+	dbg_zload_detail("zload: load_rrset: Class=%u\n", rrset_class);
 	if (!fread_wrapper(&rrset_ttl, sizeof(rrset_ttl), 1, f)) {
+		dbg_zload("zload: load_rrset: Cannot load RRSet TTL.\n");
 		if (!use_ids) {
 			knot_dname_free(&owner);
 		}
 		return NULL;
 	}
-	dbg_zload("Zone load: rrset ttl: type: %u\n", rrset_ttl);
+	dbg_zload_detail("zload: load_rrset: TTL=%u\n", rrset_ttl);
 	if (!fread_wrapper(&rdata_count, sizeof(rdata_count), 1, f)) {
 		if (!use_ids) {
 			knot_dname_free(&owner);
 		}
 		return NULL;
 	}
-	dbg_zload("Zone load: rrset load: rdata count: %u\n", rdata_count);
+	dbg_zload_detail("zload: load_rrset: rdata count=%u\n", rdata_count);
 	if (!fread_wrapper(&rrsig_count, sizeof(rrsig_count), 1, f)) {
 		if (!use_ids) {
 			knot_dname_free(&owner);
 		}
 		return NULL;
 	}
-	dbg_zload("Zone load: rrset load: type: %u\n", rrset_type);
+	dbg_zload_detail("zload: load_rrset: RRSIG count=%u\n", rrsig_count);
 
 dbg_zload_exec_detail(
 	char *name = knot_dname_to_str(owner);
-	dbg_zload("Loading RRSet owned by: %s\n",
+	dbg_zload_detail("zload: load_rrset: Loading RRSet owned by: %s.\n",
 	          name);
 	free(name);
 );
 
 	rrset = knot_rrset_new(owner, rrset_type, rrset_class, rrset_ttl);
-	
 	if (rrset == NULL) {
 		dbg_zload("zload: load_rrset: Could not create rrset.");
 		knot_dname_free(&owner);
@@ -507,7 +571,7 @@ dbg_zload_exec_detail(
 		owner = NULL;
 	}
 
-	dbg_zload("RRSet type: %d\n", rrset->type);
+	dbg_zload_detail("zload: load_rrset: RRSet type=%d\n", rrset->type);
 
 	knot_rdata_t *tmp_rdata = NULL;
 
@@ -515,7 +579,13 @@ dbg_zload_exec_detail(
 		tmp_rdata = knot_load_rdata(rrset->type, f,
 					      id_array, use_ids);
 		if (tmp_rdata) {
-			knot_rrset_add_rdata(rrset, tmp_rdata);
+			int ret = knot_rrset_add_rdata(rrset, tmp_rdata);
+			if (ret != KNOT_EOK) {
+				dbg_zload("zload: load_rrset: Cannot add "
+				          "RDATA.\n");
+				knot_rrset_deep_free(&rrset, 0, 1, 1);
+				return NULL;
+			}
 		} else {
 			knot_rrset_deep_free(&rrset, 0, 1, 1);
 			return NULL;
@@ -524,9 +594,15 @@ dbg_zload_exec_detail(
 
 	knot_rrset_t *tmp_rrsig = NULL;
 
-	dbg_zload("Reading: %d RRSIGs\n", rrsig_count);
+	dbg_zload_detail("zload: load_rrset: Reading: %d RRSIGs.\n",
+	                 rrsig_count);
 	if (rrsig_count) {
 		tmp_rrsig = knot_load_rrsig(f, id_array, use_ids);
+		if (tmp_rrsig == NULL) {
+			dbg_zload("zload: load_rrset: Cannot load RRSIG.\n");
+			knot_rrset_deep_free(&rrset, 0, 1, 1);
+			return NULL;
+		}
 		if (!use_ids) {
 			knot_rrset_set_owner(tmp_rrsig, rrset->owner);
 		}
@@ -534,8 +610,8 @@ dbg_zload_exec_detail(
 
 	knot_rrset_set_rrsigs(rrset, tmp_rrsig);
 
-	dbg_zload("Finished loading RRSet %p\n", rrset);
-
+	dbg_zload_detail("zload: load_rrset: Finished loading RRSet %p.\n",
+	                 rrset);
 	return rrset;
 }
 
@@ -548,6 +624,10 @@ dbg_zload_exec_detail(
  */
 static knot_node_t *knot_load_node(FILE *f, knot_dname_t **id_array)
 {
+	if (f == NULL || id_array == NULL) {
+		dbg_zload("zload: load_node: Wrong parameters.\n");
+		return NULL;
+	}
 	uint8_t flags = 0;
 	knot_node_t *node = NULL;
 	uint32_t parent_id = 0;
@@ -557,45 +637,51 @@ static knot_node_t *knot_load_node(FILE *f, knot_dname_t **id_array)
 
 	/* At the beginning of node - just dname_id !!!.*/
 	if (!fread_wrapper(&dname_id, sizeof(dname_id), 1, f)) {
+		dbg_zload("zload: load_node: Cannot read owner ID.\n");
 		return NULL;
 	}
 
 	if (!fread_wrapper(&parent_id, sizeof(parent_id), 1, f)) {
+		dbg_zload("zload: load_node: Cannot read owner ID.\n");
 		return NULL;
 	}
 
 	if (!fread_wrapper(&flags, sizeof(flags), 1, f)) {
+		dbg_zload("zload: load_node: Cannot read node flags.\n");
 		return NULL;
 	}
 
 	if (!fread_wrapper(&nsec3_node_id, sizeof(nsec3_node_id), 1, f)) {
+		dbg_zload("zload: load_node: Cannot read NSEC3 node.\n");
 		return NULL;
 	}
 
 	if (!fread_wrapper(&rrset_count, sizeof(rrset_count), 1, f)) {
+		dbg_zload("zload: load_node: Cannot read rrset count.\n");
 		return NULL;
 	}
 	knot_dname_t *owner = id_array[dname_id];
 
-	dbg_zload("Node owner id: %d\n", dname_id);
-	dbg_zload("Node owned by: %s\n", knot_dname_to_str(owner));
-	dbg_zload("Number of RRSets in a node: %d\n", rrset_count);
+	dbg_zload_detail("zload: load_node: Node owner id: %d.\n", dname_id);
+dbg_zload_exec_detail(
+	char *name = knot_dname_to_str(owner);
+	dbg_zload_detail("zload: load_node: Node owned by: %s.\n", name);
+	free(name);
+);
+	dbg_zload_detail("zload: load_node: Number of RRSets in a node: %d.\n",
+	                 rrset_count);
 
 	node = owner->node;
 
 	if (node == NULL) {
-		fprintf(stderr, "zone: Could not create node.\n");
+		dbg_zload("zload: load_node: NULL node, cannot proceed.\n");
 		return NULL;
 	}
 	/* XXX can it be 0, ever? I think not. */
 	if (nsec3_node_id != 0) {
 		knot_node_set_nsec3_node(node, id_array[nsec3_node_id]->node);
-		/* CLEANUP */
-//		node->nsec3_node = id_array[nsec3_node_id]->node;
 	} else {
 		knot_node_set_nsec3_node(node, NULL);
-		/* CLEANUP */
-//		node->nsec3_node = NULL;
 	}
 
 	/* Retain new owner while releasing replaced owner. */
@@ -611,7 +697,7 @@ static knot_node_t *knot_load_node(FILE *f, knot_dname_t **id_array)
 		knot_node_set_parent(node, NULL);
 	}
 
-	knot_rrset_t *tmp_rrset;
+	knot_rrset_t *tmp_rrset = NULL;
 
 	for (int i = 0; i < rrset_count; i++) {
 		if ((tmp_rrset = knot_load_rrset(f, id_array, 1)) == NULL) {
@@ -619,7 +705,7 @@ static knot_node_t *knot_load_node(FILE *f, knot_dname_t **id_array)
 			/*!< \todo #1686 
 			 * Refactor freeing, might not be enough.
 			 */
-			fprintf(stderr, "zone: Could not load rrset.\n");
+			dbg_zload("zload: load_node: Cannot load RRSet.\n");
 			return NULL;
 		}
 		/* Retain new owner while releasing replaced owner. */
@@ -628,12 +714,13 @@ static knot_node_t *knot_load_node(FILE *f, knot_dname_t **id_array)
 			knot_rrset_set_owner(tmp_rrset->rrsigs, node->owner);
 		}
 		if (knot_node_add_rrset(node, tmp_rrset, 0) < 0) {
-			fprintf(stderr, "zone: Could not add rrset.\n");
+			dbg_zload("zload: load_node: Cannot add RRSet "
+			          "to node.\n");
 			return NULL;
 		}
 	}
 	assert(node != NULL);
-	dbg_zload("Node loaded: %p\n", node);
+	dbg_zload_detail("zload: load_node: Node loaded: %p\n", node);
 	return node;
 }
 
@@ -647,6 +734,10 @@ static knot_node_t *knot_load_node(FILE *f, knot_dname_t **id_array)
 static void find_and_set_wildcard_child(knot_zone_contents_t *zone,
 				 knot_node_t *node, int nsec3)
 {
+	if (zone == NULL || node == NULL) {
+		dbg_zload("zload: set_wc: Bad arguments.\n");
+		return;
+	}
 	knot_dname_t *chopped = knot_dname_left_chop(node->owner);
 	knot_node_t *wildcard_parent;
 	if (!nsec3) {
@@ -708,7 +799,10 @@ static unsigned long calculate_crc(FILE *f)
 	unsigned char *chunk = malloc(sizeof(unsigned char) * chunk_size);
 	CHECK_ALLOC_LOG(chunk, 0);
 	while ((file_size - read_bytes) > chunk_size) {
-		if (!fread_wrapper(chunk, sizeof(unsigned char), chunk_size, f)) {
+		if (!fread_wrapper(chunk,
+		                   sizeof(unsigned char), chunk_size, f)) {
+			dbg_zload("zload: calculate_crc: Failed to read "
+			          "data chunk.\n");
 			free(chunk);
 			return 0;
 		}
@@ -720,6 +814,8 @@ static unsigned long calculate_crc(FILE *f)
 	/* Read the rest of the file */
 	if (!fread_wrapper(chunk, sizeof(unsigned char), file_size - read_bytes,
 	                f)) {
+		dbg_zload("zload: calculate_crc: Failed to read "
+		          "data remainder.\n");
 		free(chunk);
 		return 0;
 	}
@@ -737,6 +833,7 @@ int knot_zload_open(zloader_t **dst, const char *filename)
 	char crc_buf[65];
 
 	if (!dst || !filename) {
+		dbg_zload("zload: open: Bad arguments.\n");
 		return KNOT_EBADARG;
 	}
 
@@ -749,7 +846,7 @@ int knot_zload_open(zloader_t **dst, const char *filename)
 	if (unlikely(!f)) {
 		int reason = errno;
 		dbg_zload("knot_zload_open: failed to open '%s'\n",
-				   filename);
+		          filename);
 		switch (reason) {
 		case EACCES: return KNOT_EACCES; break;
 		case ENOENT: return KNOT_ENOENT; break;
@@ -765,9 +862,11 @@ int knot_zload_open(zloader_t **dst, const char *filename)
 
 	/* Read CRC from filename.crc file */
 	char *crc_path =
-		malloc(sizeof(char) * (strlen(filename) + 4 /* strlen(".crc") */ + 1));
+		malloc(sizeof(char) *
+	               (strlen(filename) + 4 /* strlen(".crc") */ + 1));
 	if (unlikely(!crc_path)) {
 		fclose(f);
+		ERR_ALLOC_FAILED;
 		return KNOT_ENOMEM;
 	}
 	memcpy(crc_path, filename, strlen(filename));
@@ -815,16 +914,7 @@ int knot_zload_open(zloader_t **dst, const char *filename)
 	/* Free some value and close the CRC file */
 	free(crc_path);
 	close(f_crc);
-
-	/* Compare calculated and read CRCs. */
-	if (crc_from_file != crc_calculated) {
-		dbg_zload("knot_zload_open: CRC failed for "
-		                   "file '%s'\n",
-		                   filename);
-		fclose(f);
-		return KNOT_ECRC;
-	}
-
+	
 	/* Check magic sequence. */
 	static const uint8_t MAGIC[MAGIC_LENGTH] = MAGIC_BYTES;
 	if (!knot_check_magic(f, MAGIC, MAGIC_LENGTH)) {
@@ -836,6 +926,15 @@ int knot_zload_open(zloader_t **dst, const char *filename)
 		return KNOT_EMALF; // Illegal byte sequence (POSIX.1, C99)
 	}
 
+	/* Compare calculated and read CRCs. */
+	if (crc_from_file != crc_calculated) {
+		dbg_zload("knot_zload_open: CRC failed for "
+		                   "file '%s'\n",
+		                   filename);
+		fclose(f);
+		return KNOT_ECRC;
+	}
+
 	/* Read source file length. */
 	uint32_t sflen = 0;
 	if (!fread_wrapper(&sflen, 1, sizeof(uint32_t), f)) {
@@ -891,43 +990,12 @@ static void cleanup_id_array(knot_dname_t **id_array,
 	free(id_array);
 }
 
-//static knot_dname_table_t *create_dname_table(FILE *f, uint max_id)
-//{
-//	if (f == NULL ) {
-//		return NULL;
-//	}
-
-//	if (!fread_wrapper(&max_id, sizeof(max_id), 1, f)) {
-//		return NULL;
-//	}
-
-//	knot_dname_table_t *dname_table = knot_dname_table_new();
-//	if (dname_table == NULL) {
-//		return NULL;
-//	}
-
-//	/* Create nodes containing dnames. */
-//	for (uint i = 1; i < max_id; i++) {
-//		knot_dname_t *loaded_dname = read_dname_with_id(f);
-//		if (loaded_dname == NULL) {
-//			knot_dname_table_deep_free(&dname_table);
-//			return NULL;
-//		}
-//		if (knot_dname_table_add_dname(dname_table,
-//		                                 loaded_dname) != KNOT_EOK) {
-
-//		}
-//	}
-
-//	return dname_table;
-//}
-
 static knot_dname_table_t *create_dname_table_from_array(
 	knot_dname_t **array, uint max_id)
 {
 	if (array == NULL) {
 		/* should I set errno or what ... ? */
-		dbg_zload("No array passed\n");
+		dbg_zload("zload: create_dname_table: No array passed.\n");
 		return NULL;
 	}
 
@@ -941,19 +1009,28 @@ static knot_dname_table_t *create_dname_table_from_array(
 		assert(array[i]);
 		if (knot_dname_table_add_dname(ret,
 						 array[i]) != KNOT_EOK) {
-			dbg_zload("Could not add: %s\n",
-			                   knot_dname_to_str(array[i]));
+dbg_zload_exec_detail(
+			char *name = knot_dname_to_str(array[i]);
+			dbg_zload_detail("zload: create_dname_table: "
+			                 "Could not add dname: %s.\n",
+			                 name);
+			free(name);
+);
+			dbg_zload("zload: create_dname_table: "
+			          "Could not add dname.\n");
 			knot_dname_table_deep_free(&ret);
 			return NULL;
 		}
 	}
 
+	dbg_zload("zload: create_dname_table: Table loaded.\n");
 	return ret;
 }
 
 static knot_dname_t **create_dname_array(FILE *f, uint max_id)
 {
 	if (f == NULL) {
+		dbg_zload("zload: create_dname_array: Wrong arguments.\n");
 		return NULL;
 	}
 
@@ -964,23 +1041,24 @@ static knot_dname_t **create_dname_array(FILE *f, uint max_id)
 		return NULL;
 	}
 
-	memset(array, 0, sizeof(knot_dname_t *) * (max_id + 1));	
+	memset(array, 0, sizeof(knot_dname_t *) * (max_id + 1));
 
 	for (uint i = 0; i < max_id - 1; i++) {
 		knot_dname_t *read_dname = read_dname_with_id(f);
 		if (read_dname == NULL) {
+			dbg_zload("zload: create_dname_array: "
+			          "Cannot read dname.\n" );
 			cleanup_id_array(array, 0, i);
 			return NULL;
 		}
 
 		if (read_dname->id < max_id) {
-
 			/* Create new node from dname. */
 			read_dname->node = knot_node_new(read_dname, NULL, 0);
-
 			if (read_dname->node == NULL) {
-				ERR_ALLOC_FAILED;
-
+				dbg_zload("zload: create_dname_array: "
+				          "Cannot create new node "
+				          "from dname.\n");
 				/* Release read dname. */
 				knot_dname_release(read_dname);
 				cleanup_id_array(array, 0, i);
@@ -991,21 +1069,24 @@ static knot_dname_t **create_dname_array(FILE *f, uint max_id)
 			array[read_dname->id] = read_dname;
 		} else {
 			/* Release read dname. */
+			dbg_zload("zload: create_dname_array: "
+			          "dname id is out of range.\n");
 			knot_dname_release(read_dname);
 			cleanup_id_array(array, 0, i);
 			return NULL;
 		}
 
 	}
-
+	
+	dbg_zload("zload: create_dname_array: Array created.\n");
 	return array;
 }
 
 knot_zone_t *knot_zload_load(zloader_t *loader)
 {
-	dbg_zload("Loading zone, loader: %p\n", loader);
+	dbg_zload("zload: load: Loading zone, loader: %p.\n", loader);
 	if (!loader) {
-		dbg_zload("NULL loader!\n");
+		dbg_zload("zload: load: NULL loader!\n");
 		return NULL;
 	}
 
@@ -1020,22 +1101,24 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 	uint32_t auth_node_count;
 
 	if (!fread_wrapper(&node_count, sizeof(node_count), 1, f)) {
-		dbg_zload("wrong read!\n");
+		dbg_zload("zload: load: Cannot read node count!\n");
 		return NULL;
 	}
 
 	if (!fread_wrapper(&nsec3_node_count, sizeof(nsec3_node_count), 1, f)) {
-		dbg_zload("wrong read!\n");
+		dbg_zload("zload: load: Cannot read NSEC3 node count!\n");
 		return NULL;
 	}
 	if (!fread_wrapper(&auth_node_count,
 	      sizeof(auth_node_count), 1, f)) {
-		dbg_zload("wrong read!\n");
+		dbg_zload("zload: load: Cannot read authoritative "
+		          "node count!\n");
 		return NULL;
 	}
-	dbg_zload("authoritative nodes: %u\n", auth_node_count);
-
-	dbg_zload("loading %u nodes\n", node_count);
+	
+	dbg_zload_verb("zload: load: Authoritative nodes: %u.\n",
+	               auth_node_count);
+	dbg_zload_verb("zload: load: :oading %u nodes.\n", node_count);
 
 	uint32_t total_dnames = 0;
 	/* First, read number of dnames in dname table. */
@@ -1043,18 +1126,19 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 		return NULL;
 	}
 
-	dbg_zload("total dname count: %d\n", total_dnames);
+	dbg_zload_verb("zload: load: Total dname count: %d.\n", total_dnames);
 
 	/* Create id array. */
 	knot_dname_t **id_array = create_dname_array(f, total_dnames);
 	if (id_array == NULL) {
+		dbg_zload("zload: load: Cannot create ID array.\n");
 		return NULL;
 	}
 
 	knot_dname_table_t *dname_table =
 		create_dname_table_from_array(id_array, total_dnames);
 	if (dname_table == NULL) {
-		ERR_ALLOC_FAILED;
+		dbg_zload("zload: load: Cannot create ID array.\n");
 		cleanup_id_array(id_array, 1, total_dnames);
 		free(dname_table);
 		return NULL;
@@ -1071,13 +1155,14 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 		return NULL;
 	}
 
-	dbg_zload("Apex node loaded: %p\n", apex);
+	dbg_zload_verb("zload: load: Apex node loaded: %p.\n", apex);
 
 	knot_zone_t *zone = knot_zone_new(apex, auth_node_count, 0);
 	if (zone == NULL) {
 		cleanup_id_array(id_array, 1,
 				 node_count + nsec3_node_count + 1);
-		dbg_zload("Failed to create new zone from apex!\n");
+		dbg_zload("zload: load: Failed to create new "
+		          "zone from apex!\n");
 		knot_node_free(&apex, 0);
 		free(dname_table);
 		return NULL;
@@ -1090,19 +1175,21 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 	contents->dname_table = dname_table;
 
 	knot_node_set_previous(apex, NULL);
-
 	knot_node_t *last_node = 0;
-
 	last_node = apex;
 
 	for (uint i = 1; i < node_count; i++) {
 		tmp_node = knot_load_node(f, id_array);
-
 		if (tmp_node != NULL) {
 			if (knot_zone_contents_add_node(contents, tmp_node,
 			                                  0, 0, 0) != 0) {
-				fprintf(stderr, "!! cannot add node\n");
-				continue;
+				cleanup_id_array(id_array, 1,
+				                 node_count +
+				                 nsec3_node_count + 1);
+				knot_zone_deep_free(&zone, 0);
+				dbg_zload("zload: load: Failed to add node "
+				          "to zone.\n");
+				return NULL;
 			}
 			if (knot_dname_is_wildcard(tmp_node->owner)) {
 				find_and_set_wildcard_child(contents,
@@ -1118,8 +1205,12 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 			}
 
 		} else {
-			fprintf(stderr, "zone: Node error (in %s).\n",
+			dbg_zload("zload: load: Node error (in %s).\n",
 				loader->filename);
+			knot_zone_deep_free(&zone, 0);
+			cleanup_id_array(id_array, node_count + 1,
+					 nsec3_node_count + 1);
+			return NULL;
 		}
 	}
 
@@ -1128,7 +1219,8 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 	knot_node_set_previous(knot_zone_contents_get_apex(contents),
 	                         last_node);
 
-	dbg_zload("loading %u nsec3 nodes\n", nsec3_node_count);
+	dbg_zload_detail("zload: load: Loading %u nsec3 nodes\n",
+	                 nsec3_node_count);
 
 	knot_node_t *nsec3_first = NULL;
 
@@ -1137,10 +1229,12 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 
 		assert(nsec3_first != NULL);
 
-		if (knot_zone_contents_add_nsec3_node(contents, nsec3_first, 0, 0, 0)
+		if (knot_zone_contents_add_nsec3_node(contents, nsec3_first,
+		                                      0, 0, 0)
 		    != 0) {
-			fprintf(stderr, "!! cannot add first nsec3 node, "
-				"exiting.\n");
+			dbg_zload("zload: load: "
+			          "cannot add first nsec3 node, "
+			          "exiting.\n");
 			knot_zone_deep_free(&zone, 0);
 			cleanup_id_array(id_array, node_count + 1,
 					 nsec3_node_count + 1);
@@ -1157,8 +1251,12 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 		if (tmp_node != NULL) {
 			if (knot_zone_contents_add_nsec3_node(contents,
 			    tmp_node, 0, 0, 0) != 0) {
-				fprintf(stderr, "!! cannot add nsec3 node\n");
-				continue;
+				dbg_zload("zload: load: Cannot add "
+				          "NSEC3 node.\n");
+				knot_zone_deep_free(&zone, 0);
+				cleanup_id_array(id_array, node_count + 1,
+						 nsec3_node_count + 1);
+				return NULL;
 			}
 
 			knot_node_set_previous(tmp_node, last_node);
@@ -1182,8 +1280,7 @@ knot_zone_t *knot_zload_load(zloader_t *loader)
 	}
 	free(id_array);
 
-	dbg_zload("zone loaded, returning: %p\n", zone);
-	
+	dbg_zload("zload: load: Zone loaded, returning: %p,\n", zone);
 	return zone;
 }
 
@@ -1231,6 +1328,7 @@ int knot_zload_rrset_deserialize(knot_rrset_t **rrset,
                                    uint8_t *stream, size_t *size)
 {
 	if (stream == NULL || size == 0) {
+		dbg_zload("zload: rrset_deserialize: Bad arguments.\n");
 		return KNOT_EBADARG;
 	}
 
@@ -1240,8 +1338,8 @@ int knot_zload_rrset_deserialize(knot_rrset_t **rrset,
 	knot_zload_stream_remaining = knot_zload_stream_size = *size;
 
 	knot_rrset_t *ret = knot_load_rrset(NULL, NULL, 0);
-
 	if (ret == NULL) {
+		dbg_zload("zload: rrset_deserialize: Cannot load RRSet.\n");
 		knot_zload_stream = NULL;
 		knot_zload_stream_remaining = 0;
 		knot_zload_stream_size = 0;
@@ -1254,7 +1352,9 @@ int knot_zload_rrset_deserialize(knot_rrset_t **rrset,
 	knot_zload_stream = NULL;
 	knot_zload_stream_remaining = 0;
 	knot_zload_stream_size = 0;
-
+	
+	dbg_zload_detail("zload: rrset_deserialize: RRSet deserialized "
+	                 "successfully.\n");
 	return KNOT_EOK;
 }
 
