@@ -821,17 +821,19 @@ void ck_destroy_table(ck_hash_table_t **table, void (*dtor_value)(void *value),
 	while (item != NULL) {
 		// disconnect the item
 		(*table)->stash = item->next;
-		/*! \todo Investigate this. */
-		assert(item->item != NULL);
 
-		if (dtor_value) {
-			dtor_value(item->item->value);
-		}
-		if (delete_key) {
-			free((void *)item->item->key);
+		/*! \todo Check item semantics. (#1688) */
+		if (item->item != NULL) {
+			if (dtor_value) {
+				dtor_value(item->item->value);
+			}
+			if (delete_key) {
+				free((void *)item->item->key);
+			}
+
+			free((void *)item->item);
 		}
 
-		free((void *)item->item);
 		free(item);
 		item = (*table)->stash;
 	}
@@ -1035,6 +1037,7 @@ int ck_update_item(const ck_hash_table_t *table, const char *key, size_t length,
 	ck_hash_table_item_t **item = ck_find_item_nc(table, key, length);
 
 	if (item == NULL || (*item) == NULL) {
+		rcu_read_unlock();
 		return -1;
 	}
 
@@ -1058,6 +1061,7 @@ int ck_delete_item(const ck_hash_table_t *table, const char *key, size_t length,
 	ck_hash_table_item_t **place = ck_find_item_nc(table, key, length);
 
 	if (place == NULL) {
+		rcu_read_unlock();
 		return -1;
 	}
 
