@@ -87,6 +87,7 @@ typedef struct knot_ns_xfr {
 	knot_rcode_t rcode;
 	xfr_callback_t send;
 	int session;
+	int session_closed;
 	
 	/*!
 	 * XFR-out: Output buffer.
@@ -156,6 +157,11 @@ enum knot_ns_xfr_flag_t {
 	XFR_FLAG_AXFR_FINISHED = 1 << 2
 };
 
+typedef enum knot_ns_transport {
+	NS_TRANSPORT_UDP = 1 << 0,
+	NS_TRANSPORT_TCP = 1 << 1
+} knot_ns_transport_t;
+
 /*!
  * \brief XFR request types.
  */
@@ -225,8 +231,15 @@ int knot_ns_parse_packet(const uint8_t *query_wire, size_t qsize,
  * \param response_wire Place for wire format of the response.
  * \param rsize Size of the error response will be stored here.
  */
-void knot_ns_error_response(const knot_nameserver_t *nameserver, uint16_t query_id,
-                       uint8_t rcode, uint8_t *response_wire, size_t *rsize);
+void knot_ns_error_response(const knot_nameserver_t *nameserver,
+                            uint16_t query_id, uint8_t *flags1_query,
+                            uint8_t rcode, uint8_t *response_wire,
+                            size_t *rsize);
+
+int knot_ns_error_response_from_query(const knot_nameserver_t *nameserver,
+                                      const uint8_t *query, size_t size,
+                                      uint8_t rcode, uint8_t *response_wire,
+                                      size_t *rsize);
 
 void knot_ns_error_response_full(knot_nameserver_t *nameserver,
                                  knot_packet_t *response, uint8_t rcode,
@@ -234,7 +247,7 @@ void knot_ns_error_response_full(knot_nameserver_t *nameserver,
 
 int knot_ns_prep_normal_response(knot_nameserver_t *nameserver,
                                  knot_packet_t *query, knot_packet_t **resp,
-                                 const knot_zone_t **zone);
+                                 const knot_zone_t **zone, size_t max_size);
 
 /*!
  * \brief Creates a response for the given normal query using the data of the
