@@ -110,6 +110,19 @@ static void knot_packet_init_pointers_response(knot_packet_t *pkt)
 	dbg_packet("Compression offsets: %p\n", pkt->compression.offsets);
 
 	pkt->compression.max = DEFAULT_DOMAINS_IN_RESPONSE;
+	pkt->compression.default_count = DEFAULT_DOMAINS_IN_RESPONSE;
+
+	// wildcard nodes and SNAMEs associated with them
+	pkt->wildcard_nodes.nodes = (const knot_node_t **)pos;
+	pos += DEFAULT_WILDCARD_NODES * sizeof(const knot_node_t *);
+	pkt->wildcard_nodes.snames = (const knot_dname_t **)pos;
+	pos += DEFAULT_WILDCARD_NODES * sizeof(knot_dname_t *);
+
+	dbg_packet("Wildcard nodes: %p\n", pkt->wildcard_nodes.nodes);
+	dbg_packet("Wildcard SNAMEs: %p\n", pkt->wildcard_nodes.snames);
+
+	pkt->wildcard_nodes.default_count = DEFAULT_WILDCARD_NODES;
+	pkt->wildcard_nodes.max = DEFAULT_WILDCARD_NODES;
 
 	pkt->tmp_rrsets = (const knot_rrset_t **)pos;
 	pos += DEFAULT_TMP_RRSETS * sizeof(const knot_rrset_t *);
@@ -633,11 +646,16 @@ static void knot_packet_free_allocated_space(knot_packet_t *pkt)
 		free(pkt->additional);
 	}
 
-	if (pkt->compression.max > DEFAULT_DOMAINS_IN_RESPONSE) {
+	if (pkt->compression.max > pkt->compression.default_count) {
 		free(pkt->compression.dnames);
 		free(pkt->compression.offsets);
 	}
 
+	if (pkt->wildcard_nodes.max > pkt->wildcard_nodes.default_count) {
+		free(pkt->wildcard_nodes.nodes);
+		free(pkt->wildcard_nodes.snames);
+	}
+
 	if (pkt->tmp_rrsets_max > DEFAULT_RRSET_COUNT(TMP_RRSETS, pkt)) {
 		free(pkt->tmp_rrsets);
 	}
@@ -1459,6 +1477,10 @@ void knot_packet_free(knot_packet_t **packet)
 	dbg_packet("Freeing tmp RRSets...\n");
 	knot_packet_free_tmp_rrsets(*packet);
 
+	/*! \note The above code will free the domain names pointed to by
+	 *        the list of wildcard nodes. It should not matter, however.
+	 */
+
 	// check if some additional space was allocated for the packet
 	dbg_packet("Freeing additional allocated space...\n");
 	knot_packet_free_allocated_space(*packet);
@@ -1470,6 +1492,9 @@ void knot_packet_free(knot_packet_t **packet)
 		free((*packet)->wireformat);
 	}
 
+	// free EDNS options
+	knot_edns_free_options(&(*packet)->opt_rr);
+
 	dbg_packet("Freeing packet structure\n");
 	free(*packet);
 	*packet = NULL;
@@ -1499,7 +1524,7 @@ void knot_packet_dump(const knot_packet_t *packet)
 	dbg_packet("DNS packet:\n-----------------------------\n");
 
 	dbg_packet("\nHeader:\n");
-	dbg_packet("  ID: %u", packet->header.id);
+	dbg_packet("  ID: %u\n", packet->header.id);
 	dbg_packet("  FLAGS: %s %s %s %s %s %s %s\n",
 	       knot_wire_flags_get_qr(packet->header.flags1) ? "qr" : "",
 	       knot_wire_flags_get_aa(packet->header.flags1) ? "aa" : "",
