@@ -96,7 +96,8 @@ void knot_response_clear(knot_packet_t *resp, int clear_question);
  */
 int knot_response_add_opt(knot_packet_t *resp,
                             const knot_opt_rr_t *opt_rr,
-                            int override_max_size);
+                            int override_max_size,
+                            int add_nsid);
 
 /*!
  * \brief Adds a RRSet to the Answer section of the response.
@@ -196,6 +197,10 @@ void knot_response_set_tc(knot_packet_t *response);
 int knot_response_add_nsid(knot_packet_t *response, const uint8_t *data,
                              uint16_t length);
 
+int knot_response_add_wildcard_node(knot_packet_t *response,
+                                    const knot_node_t *node,
+                                    const knot_dname_t *sname);
+
 #endif /* _KNOT_response_H_ */
 
 /*! @} */
