@@ -32,7 +32,7 @@
 /*! \brief TSIG algorithms table. */
 #define TSIG_ALG_TABLE_SIZE 8
 static knot_lookup_table_t tsig_alg_table[TSIG_ALG_TABLE_SIZE] = {
-	{ KNOT_TSIG_ALG_GSS_TSIG, "gss-tsig." },
+	{ KNOT_TSIG_ALG_NULL, "gss-tsig." },
 	{ KNOT_TSIG_ALG_HMAC_MD5, "hmac-md5.sig-alg.reg.int." },
 	{ KNOT_TSIG_ALG_HMAC_SHA1, "hmac-sha1." },
 	{ KNOT_TSIG_ALG_HMAC_SHA224, "hmac-sha224." },
@@ -84,6 +84,9 @@ int tsig_rdata_set_alg_name(knot_rrset_t *tsig, knot_dname_t *alg_name)
 	}
 
 	knot_rdata_item_set_dname(rdata, 0, alg_name_copy);
+	
+	/* Release the dname. We want it to have 1 reference only. */
+	knot_dname_release(alg_name_copy);
 
 	return KNOT_EOK;
 }
@@ -103,12 +106,15 @@ int tsig_rdata_set_alg(knot_rrset_t *tsig, tsig_algorithm_t alg)
 	const char *alg_str = tsig_alg_to_str(alg);
 	knot_dname_t *alg_name_copy = knot_dname_new_from_str(alg_str,
 							      strlen(alg_str),
-							      0);
+							      NULL);
 	if (!alg_name_copy) {
 		return KNOT_ENOMEM;
 	}
-
+	
 	knot_rdata_item_set_dname(rdata, 0, alg_name_copy);
+	
+	/* Release the dname. We want it to have 1 reference only. */
+	knot_dname_release(alg_name_copy);
 
 	return KNOT_EOK;
 }
@@ -306,8 +312,32 @@ const knot_dname_t *tsig_rdata_alg_name(const knot_rrset_t *tsig)
 
 tsig_algorithm_t tsig_rdata_alg(const knot_rrset_t *tsig)
 {
-	/*! \todo [TSIG] Implement me. */
-	return KNOT_TSIG_ALG_HMAC_MD5;
+	if (!tsig) {
+		return KNOT_TSIG_ALG_NULL;
+	}
+
+	/* Get the algorithm name. */
+	const knot_dname_t *alg_name = tsig_rdata_alg_name(tsig);
+	if (!alg_name) {
+		dbg_tsig_detail("TSIG: rdata: cannot get algorithm name.\n");
+		return KNOT_TSIG_ALG_NULL;
+	}
+
+	/* Convert alg name to string. */
+	char *name = knot_dname_to_str(alg_name);
+	if (!name) {
+		dbg_tsig_detail("TSIG: rdata: cannot convert alg name.\n");
+		return KNOT_TSIG_ALG_NULL;
+	}
+
+	knot_lookup_table_t *item = knot_lookup_by_name(tsig_alg_table, name);
+	free(name);
+	if (!item) {
+		dbg_tsig_detail("TSIG: rdata: unknown algorithm.\n");
+		return KNOT_TSIG_ALG_NULL;
+	}
+
+	return item->id;
 }
 
 uint64_t tsig_rdata_time_signed(const knot_rrset_t *tsig)
@@ -516,6 +546,8 @@ uint16_t tsig_alg_digest_length(tsig_algorithm_t alg)
 		case KNOT_TSIG_ALG_HMAC_SHA224:
 			return KNOT_TSIG_ALG_DIG_LENGTH_SHA224;
 		case KNOT_TSIG_ALG_HMAC_SHA256:
+			return KNOT_TSIG_ALG_DIG_LENGTH_SHA256;
+		case KNOT_TSIG_ALG_HMAC_SHA384:
 			return KNOT_TSIG_ALG_DIG_LENGTH_SHA384;
 		case KNOT_TSIG_ALG_HMAC_SHA512:
 			return KNOT_TSIG_ALG_DIG_LENGTH_SHA512;
@@ -581,7 +613,7 @@ const char* tsig_alg_to_str(tsig_algorithm_t alg)
 size_t tsig_wire_maxsize(const knot_key_t* key)
 {
 	size_t alg_name_size = strlen(tsig_alg_to_str(key->algorithm)) + 1;
-		
+
 	return knot_dname_size(key->name) +
 	sizeof(uint16_t) + /* TYPE */
 	sizeof(uint16_t) + /* CLASS */
