@@ -730,7 +730,7 @@ dbg_xfrin_exec(
 				return KNOT_ERROR;  /*! \todo Other error code. */
 			} else if (ret == 1) {
 				assert(node != NULL);
-dbg_xfrin_exec(
+dbg_xfrin_exec_verb(
 				char *name = knot_dname_to_str(node->owner);
 				dbg_xfrin("Found node for the record in "
 					   "zone: %s. Merged.\n", name);
@@ -744,7 +744,7 @@ dbg_xfrin_exec(
 //				knot_rrset_deep_free(&rr, 1, 1, 1);
 			} else {
 				assert(node != NULL);
-dbg_xfrin_exec(
+dbg_xfrin_exec_verb(
 				char *name = knot_dname_to_str(node->owner);
 				dbg_xfrin("Found node for the record in "
 					       "zone: %s.\n", name);
@@ -1113,10 +1113,10 @@ int xfrin_process_ixfr_packet(knot_ns_xfr_t *xfr)
 	/*! \todo This may be implemented with much less IFs! */
 	
 	while (ret == KNOT_EOK && rr != NULL) {
-dbg_xfrin_exec(
-		dbg_xfrin("Next loop, state: %d\n", state);
+dbg_xfrin_exec_verb(
+		dbg_xfrin_verb("Next loop, state: %d\n", state);
 		char *name = knot_dname_to_str(knot_rrset_owner(rr));
-		dbg_xfrin("Actual RR: %s, type %s.\n", name, 
+		dbg_xfrin_verb("Actual RR: %s, type %s.\n", name,
 		               knot_rrtype_to_string(knot_rrset_type(rr)));
 		free(name);
 );
@@ -1228,9 +1228,9 @@ dbg_xfrin_exec(
 		}
 		
 		// parse the next RR
-		dbg_xfrin("Parsing next RR..\n");
+		dbg_xfrin_verb("Parsing next RR..\n");
 		ret = knot_packet_parse_next_rr_answer(packet, &rr);
-		dbg_xfrin("Returned %d, %p.\n", ret, rr);
+		dbg_xfrin_verb("Returned %d, %p.\n", ret, rr);
 	}
 	
 	/*! \note Check TSIG, we're at the end of packet. It may not be 
@@ -1269,13 +1269,12 @@ cleanup:
 static int xfrin_changes_check_rrsets(knot_rrset_t ***rrsets,
                                       int *count, int *allocated, int to_add)
 {
-	/* Ensure at least requested size is allocated. */
-	int new_count = (*count + to_add);
-	assert(new_count >= 0);
-	if (new_count <= *allocated) {
+	if (*count + to_add <= *allocated) {
 		return KNOT_EOK;
 	}
 
+	int new_count = (*allocated == 0) ? 2 : *allocated * 2;
+
 	/* Allocate new memory block. */
 	knot_rrset_t **rrsets_new = malloc(new_count * sizeof(knot_rrset_t *));
 	if (rrsets_new == NULL) {
@@ -1303,12 +1302,12 @@ static int xfrin_changes_check_nodes(knot_node_t ***nodes,
 	assert(count != NULL);
 	assert(allocated != 0);
 
-	/* Ensure at least count and some reserve is allocated. */
-	int new_count = *count + 2;
-	if (new_count <= *allocated) {
+	if (*count + 2 <= *allocated) {
 		return KNOT_EOK;
 	}
 
+	int new_count = (*allocated == 0) ? 2 : *allocated * 2;
+
 	/* Allocate new memory block. */
 	const size_t node_len = sizeof(knot_node_t *);
 	knot_node_t **nodes_new = malloc(new_count * node_len);
@@ -1333,13 +1332,12 @@ static int xfrin_changes_check_nodes(knot_node_t ***nodes,
 static int xfrin_changes_check_rdata(knot_rdata_t ***rdatas, uint **types,
                                      int count, int *allocated, int to_add)
 {
-	/* Ensure at least requested size is allocated. */
-	int new_count = (count + to_add);
-	assert(new_count >= 0);
-	if (new_count <= *allocated) {
+	if (count + to_add <= *allocated) {
 		return KNOT_EOK;
 	}
 
+	int new_count = (*allocated == 0) ? 2 : *allocated * 2;
+
 	/* Allocate new memory block. */
 	knot_rdata_t **rdatas_new = malloc(new_count * sizeof(knot_rdata_t *));
 	if (rdatas_new == NULL) {
@@ -1374,25 +1372,32 @@ static int xfrin_changes_check_rdata(knot_rdata_t ***rdatas, uint **types,
 static void xfrin_changes_add_rdata(knot_rdata_t **rdatas, uint *types,
                                     int *count, knot_rdata_t *rdata, uint type)
 {
-	dbg_xfrin_detail("Adding RDATA to new RDATA list: %p\n", rdata);
+	dbg_xfrin_detail("Adding RDATA to RDATA list: %p\n", rdata);
 
-dbg_xfrin_exec_detail(
-	// try to find the first RDATA in the given list
-	for (int i = 0; i < *count; ++i) {
-		knot_rdata_t *r = rdatas[i];
-		while (r->next != rdatas[i]) {
-			if (r == rdata) {
-				dbg_xfrin_detail("Found same RDATA: %p\n", rdata);
-				knot_rdata_dump(rdata, type, 0);
-			}
-			r = r->next;
-		}
-		if (r == rdata) {
-			dbg_xfrin_detail("Found same RDATA: %p\n", rdata);
-			knot_rdata_dump(rdata, type, 0);
-		}
+	if (rdata == NULL) {
+		return;
 	}
-);
+
+//dbg_xfrin_exec_detail(
+//	// try to find the first RDATA in the given list
+//	for (int i = 0; i < *count; ++i) {
+//		knot_rdata_t *r = rdatas[i];
+//		if (r == NULL) {
+//			continue;
+//		}
+//		while (r != NULL && r->next != rdatas[i]) {
+//			if (r == rdata) {
+//				dbg_xfrin_detail("Found same RDATA: %p\n", rdata);
+//				knot_rdata_dump(rdata, type, 0);
+//			}
+//			r = r->next;
+//		}
+//		if (r == rdata) {
+//			dbg_xfrin_detail("Found same RDATA: %p\n", rdata);
+//			knot_rdata_dump(rdata, type, 0);
+//		}
+//	}
+//);
 
 	rdatas[*count] = rdata;
 	types[*count] = type;
@@ -1454,6 +1459,7 @@ static knot_rdata_t *xfrin_remove_rdata(knot_rrset_t *from,
 static int xfrin_copy_old_rrset(knot_rrset_t *old, knot_rrset_t **copy,
                                 knot_changes_t *changes)
 {
+	dbg_xfrin_verb("Copying old RRSet: %p\n", old);
 	// create new RRSet by copying the old one
 //	int ret = knot_rrset_shallow_copy(old, copy);
 	int ret = knot_rrset_deep_copy(old, copy);
@@ -1486,6 +1492,7 @@ static int xfrin_copy_old_rrset(knot_rrset_t *old, knot_rrset_t **copy,
 
 	changes->new_rrsets[changes->new_rrsets_count++] = *copy;
 
+	dbg_xfrin_verb("Adding RDATA from the RRSet copy to new RDATA list.\n");
 	xfrin_changes_add_rdata(changes->new_rdata, changes->new_rdata_types,
 	                        &changes->new_rdata_count,
 	                        knot_rrset_get_rdata(*copy),
@@ -1495,6 +1502,8 @@ static int xfrin_copy_old_rrset(knot_rrset_t *old, knot_rrset_t **copy,
 		assert(old->rrsigs != NULL);
 		changes->new_rrsets[changes->new_rrsets_count++] =
 		                (*copy)->rrsigs;
+		dbg_xfrin_verb("Adding RDATA from RRSIG of the RRSet copy to "
+		               "new RDATA list.\n");
 		xfrin_changes_add_rdata(changes->new_rdata,
 		                        changes->new_rdata_types,
 		                        &changes->new_rdata_count,
@@ -1523,6 +1532,7 @@ static int xfrin_copy_old_rrset(knot_rrset_t *old, knot_rrset_t **copy,
 
 	changes->old_rrsets[changes->old_rrsets_count++] = old;
 
+	dbg_xfrin_verb("Adding RDATA from old RRSet to old RDATA list.\n");
 	xfrin_changes_add_rdata(changes->old_rdata, changes->old_rdata_types,
 	                        &changes->old_rdata_count, old->rdata,
 	                        knot_rrset_type(old));
@@ -1531,6 +1541,8 @@ static int xfrin_copy_old_rrset(knot_rrset_t *old, knot_rrset_t **copy,
 		assert(old->rrsigs != NULL);
 		changes->old_rrsets[changes->old_rrsets_count++] = old->rrsigs;
 
+		dbg_xfrin_verb("Adding RDATA from RRSIG of the old RRSet to "
+		               "old RDATA list.\n");
 		xfrin_changes_add_rdata(changes->old_rdata,
 		                        changes->old_rdata_types,
 		                        &changes->old_rdata_count,
@@ -1546,10 +1558,20 @@ static int xfrin_copy_old_rrset(knot_rrset_t *old, knot_rrset_t **copy,
 static int xfrin_copy_rrset(knot_node_t *node, knot_rr_type_t type,
                             knot_rrset_t **rrset, knot_changes_t *changes)
 {
+dbg_xfrin_exec_verb(
+	char *name = knot_dname_to_str(knot_node_owner(node));
+	dbg_xfrin_verb("Removing RRSet of type %s from node %s (%p)\n",
+	               knot_rrtype_to_string(type), name, node);
+	free(name);
+);
 	knot_rrset_t *old = knot_node_remove_rrset(node, type);
 
+	dbg_xfrin_verb("Removed RRSet: %p\n", old);
+	dbg_xfrin_detail("Other RRSet of the same type in the node: %p\n",
+	                 knot_node_rrset(node, type));
+
 	if (old == NULL) {
-		dbg_xfrin("RRSet not found for RR to be removed.\n");
+		dbg_xfrin_verb("RRSet not found for RR to be removed.\n");
 		return 1;
 	}
 
@@ -1558,7 +1580,7 @@ static int xfrin_copy_rrset(knot_node_t *node, knot_rr_type_t type,
 		return ret;
 	}
 	
-	dbg_xfrin_detail("Copied old rrset %p to new %p.\n", old, *rrset);
+	dbg_xfrin_verb("Copied old rrset %p to new %p.\n", old, *rrset);
 	
 	// replace the RRSet in the node copy by the new one
 	ret = knot_node_add_rrset(node, *rrset, 0);
@@ -1601,6 +1623,7 @@ static int xfrin_apply_remove_rrsigs(knot_changes_t *changes,
 			knot_rrset_rdata(remove));
 		
 		// copy the rrset
+		dbg_xfrin_verb("Copying RRSet that carries the RRSIGs.\n");
 		ret = xfrin_copy_rrset(node, type, rrset, changes);
 		if (ret != KNOT_EOK) {
 			dbg_xfrin("Failed to copy rrset from changeset.\n");
@@ -1615,10 +1638,12 @@ static int xfrin_apply_remove_rrsigs(knot_changes_t *changes,
 		// this RRSet should be the already copied RRSet so we may
 		// update it right away
 		/*! \todo Does this case even occur? */
+		dbg_xfrin_verb("Using RRSet from previous iteration.\n");
 	}
 	
 	// get the old rrsigs
 	knot_rrset_t *old = knot_rrset_get_rrsigs(*rrset);
+	dbg_xfrin_verb("Old RRSIGs from RRSet: %p\n", old);
 	if (old == NULL) {
 		return 1;
 	}
@@ -1630,8 +1655,10 @@ static int xfrin_apply_remove_rrsigs(knot_changes_t *changes,
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
+		dbg_xfrin_verb("Copied RRSIGs: %p\n", rrsigs);
 	} else {
 		rrsigs = old;
+		dbg_xfrin_verb("Using old RRSIGs: %p\n", rrsigs);
 	}
 	
 	// set the RRSIGs to the new RRSet copy
@@ -1755,11 +1782,11 @@ static int xfrin_apply_remove_normal(knot_changes_t *changes,
 	} /*! \todo Does some other case even occur? */
 	
 	if (*rrset == NULL) {
-		dbg_xfrin("RRSet not found for RR to be removed.\n");
+		dbg_xfrin_verb("RRSet not found for RR to be removed.\n");
 		return 1;
 	}
 	
-dbg_xfrin_exec(
+dbg_xfrin_exec_verb(
 	char *name = knot_dname_to_str(knot_rrset_owner(*rrset));
 	dbg_xfrin("Updating RRSet with owner %s, type %s\n", name,
 		  knot_rrtype_to_string(knot_rrset_type(*rrset)));
@@ -1932,19 +1959,27 @@ static int xfrin_apply_add_normal(knot_changes_t *changes,
 
 	int ret;
 
-	dbg_xfrin("applying rrset:\n");
+	dbg_xfrin_verb("applying rrset:\n");
 	knot_rrset_dump(add, 0);
 	
-	if (!*rrset
-	    || knot_dname_compare(knot_rrset_owner(*rrset),
-	                          knot_node_owner(node)) != 0
-	    || knot_rrset_type(*rrset)
-	       != knot_rrset_type(add)) {
-		dbg_xfrin("Removing rrset!\n");
-		*rrset = knot_node_remove_rrset(node, knot_rrset_type(add));
-	}
+	/*! \note Reusing RRSet from previous function caused it not to be
+	 *        removed from the node.
+	 *        Maybe modification of the code would allow reusing the RRSet
+	 *        as in apply_add_rrsigs() - the RRSet should not be copied
+	 *        in such case.
+	 */
+//	if (!*rrset
+//	    || knot_dname_compare(knot_rrset_owner(*rrset),
+//	                          knot_node_owner(node)) != 0
+//	    || knot_rrset_type(*rrset)
+//	       != knot_rrset_type(add)) {
+//		dbg_xfrin("Removing rrset!\n");
+//		*rrset = knot_node_remove_rrset(node, knot_rrset_type(add));
+//	}
+
+	*rrset = knot_node_remove_rrset(node, knot_rrset_type(add));
 	
-	dbg_xfrin("Removed RRSet: \n");
+	dbg_xfrin_verb("Removed RRSet: \n");
 	knot_rrset_dump(*rrset, 1);
 
 	if (*rrset == NULL) {
@@ -1980,7 +2015,7 @@ dbg_xfrin_exec_verb(
 
 	knot_rrset_t *old = *rrset;
 
-dbg_xfrin_exec(
+dbg_xfrin_exec_verb(
 	char *name = knot_dname_to_str(knot_rrset_owner(*rrset));
 	dbg_xfrin("Found RRSet with owner %s, type %s\n", name,
 	          knot_rrtype_to_string(knot_rrset_type(*rrset)));
@@ -1992,6 +2027,8 @@ dbg_xfrin_exec(
 		return ret;
 	}
 
+	dbg_xfrin_verb("Copied RRSet: %p\n", *rrset);
+
 //	dbg_xfrin("After copy: Found RRSet with owner %s, type %s\n",
 //	               knot_dname_to_str((*rrset)->owner),
 //	          knot_rrtype_to_string(knot_rrset_type(*rrset)));
@@ -2007,20 +2044,27 @@ dbg_xfrin_exec(
 	 *
 	 * TODO: add the 'add' rrset to list of old RRSets?
 	 */
-	dbg_xfrin("Merging RRSets with owners: %s %s types: %d %d\n",
-	       (*rrset)->owner->name, add->owner->name, (*rrset)->type,
-	                add->type);
+	dbg_xfrin_verb("Merging RRSets with owners: %s, %s types: %s, %s\n",
+	               (*rrset)->owner->name, add->owner->name,
+	               knot_rrtype_to_string((*rrset)->type),
+	               knot_rrtype_to_string(add->type));
 	dbg_xfrin_detail("RDATA in RRSet1: %p, RDATA in RRSet2: %p\n",
 	                 (*rrset)->rdata, add->rdata);
+
 	ret = knot_rrset_merge((void **)rrset, (void **)&add);
 	if (ret != KNOT_EOK) {
 		dbg_xfrin("Failed to merge changeset RRSet to copy.\n");
 		return KNOT_ERROR;
 	}
-	dbg_xfrin("Merge returned: %d\n", ret);
+	dbg_xfrin_verb("Merge returned: %d\n", ret);
 	knot_rrset_dump(*rrset, 1);
 	ret = knot_node_add_rrset(node, *rrset, 0);
 
+	if (ret < 0) {
+		dbg_xfrin("Failed to add merged RRSet to the node.\n");
+		return ret;
+	}
+
 	// return 2 so that the add RRSet is removed from
 	// the changeset (and thus not deleted)
 	// and put to list of new RRSets (is this ok?)
@@ -2048,7 +2092,7 @@ static int xfrin_apply_add_rrsig(knot_changes_t *changes,
 	knot_rr_type_t type = knot_rdata_rrsig_type_covered(
 	                                               knot_rrset_rdata(add));
 	
-dbg_xfrin_exec(
+dbg_xfrin_exec_verb(
 	char *name = knot_dname_to_str(knot_rrset_owner(add));
 	const char *typestr = knot_rrtype_to_string(type);
 	dbg_xfrin("Adding RRSIG: Owner %s, type covered %s.\n",
@@ -2058,6 +2102,9 @@ dbg_xfrin_exec(
 
 	int copied = 0;
 
+	/*! \note Here the check is OK, because if we aready have the RRSet,
+	 *        it's a copied one, so it is OK to modify it right away.
+	 */
 	if (!*rrset
 	    || knot_dname_compare(knot_rrset_owner(*rrset),
 	                          knot_node_owner(node)) != 0
@@ -2079,9 +2126,8 @@ dbg_xfrin_exec(
 	}
 
 	if (*rrset == NULL) {
-		dbg_xfrin("RRSet to be added not found in zone.\n");
+		dbg_xfrin_verb("RRSet to be added not found in zone.\n");
 		
-		dbg_xfrin("Creating new RRSet for RRSIG.\n");
 		// create a new RRSet to add the RRSIGs into
 		*rrset = knot_rrset_new(knot_node_get_owner(node), type,
 		                        knot_rrset_class(add),
@@ -2090,6 +2136,7 @@ dbg_xfrin_exec(
 			dbg_xfrin("Failed to create new RRSet for RRSIGs.\n");
 			return KNOT_ENOMEM;
 		}
+		dbg_xfrin_verb("Created new RRSet for RRSIG: %p.\n", *rrset);
 
 		// add the RRset to the list of new RRsets
 		ret = xfrin_changes_check_rrsets(
@@ -2116,7 +2163,7 @@ dbg_xfrin_exec(
 		changes->new_rrsets[changes->new_rrsets_count++] = *rrset;
 	}
 
-dbg_xfrin_exec(
+dbg_xfrin_exec_verb(
 		char *name = knot_dname_to_str(knot_rrset_owner(*rrset));
 		dbg_xfrin("Found RRSet with owner %s, type %s\n", name,
 			  knot_rrtype_to_string(knot_rrset_type(*rrset)));
@@ -2125,7 +2172,7 @@ dbg_xfrin_exec(
 
 	if (knot_rrset_rrsigs(*rrset) == NULL) {
 
-		dbg_xfrin("Adding new RRSIGs to RRSet.\n");
+		dbg_xfrin_verb("Adding new RRSIGs to RRSet.\n");
 		ret = knot_zone_contents_add_rrsigs(contents, add, rrset, &node,
 		                                    KNOT_RRSET_DUPL_SKIP, 1);
 
@@ -2156,7 +2203,7 @@ dbg_xfrin_exec(
 		knot_rrset_set_rrsigs(*rrset, rrsig);
 	
 		// merge the changeset RRSet to the copy
-		dbg_xfrin("Merging RRSIG to the one in the RRSet.\n");
+		dbg_xfrin_verb("Merging RRSIG to the one in the RRSet.\n");
 		ret = knot_rrset_merge((void **)&rrsig, (void **)&add);
 		if (ret != KNOT_EOK) {
 			dbg_xfrin("Failed to merge changeset RRSIG to copy: %s"
@@ -2366,6 +2413,8 @@ void xfrin_rollback_update(knot_zone_contents_t *old_contents,
 {
 	assert(changes != NULL);
 
+	dbg_xfrin("Rolling back changeset application.\n");
+
 	if (*changes != NULL) {
 		// discard new RRSets
 		for (int i = 0; i < (*changes)->new_rrsets_count; ++i) {
@@ -2448,6 +2497,17 @@ static int xfrin_apply_remove2(knot_zone_contents_t *contents,
 	int is_nsec3 = 0;
 
 	for (int i = 0; i < chset->remove_count; ++i) {
+dbg_xfrin_exec_verb(
+		char *name = knot_dname_to_str(
+			knot_rrset_owner(chset->remove[i]));
+		dbg_xfrin_verb("Removing RRSet: %s, type %s\n", name,
+                               knot_rrtype_to_string(
+		                knot_rrset_type(chset->remove[i])));
+		free(name);
+);
+dbg_xfrin_exec_detail(
+		knot_rrset_dump(chset->remove[i], 0);
+);
 
 		is_nsec3 = 0;
 
@@ -2473,8 +2533,8 @@ static int xfrin_apply_remove2(knot_zone_contents_t *contents,
 				            knot_rrset_owner(chset->remove[i]));
 			}
 			if (node == NULL) {
-				dbg_xfrin("Node not found for RR to be removed"
-				          "!\n");
+				dbg_xfrin_verb("Node not found for RR to be "
+				               "removed!\n");
 				continue;
 			}
 		}
@@ -2499,7 +2559,7 @@ static int xfrin_apply_remove2(knot_zone_contents_t *contents,
 			                                node, &rrset);
 		}
 
-		dbg_xfrin("xfrin_apply_remove() ret = %d\n", ret);
+		dbg_xfrin_verb("xfrin_apply_remove() ret = %d\n", ret);
 
 		if (ret > 0) {
 			continue;
@@ -2524,8 +2584,17 @@ static int xfrin_apply_add2(knot_zone_contents_t *contents,
 	int is_nsec3 = 0;
 
 	for (int i = 0; i < chset->add_count; ++i) {
-		dbg_xfrin_detail("Adding RRSet:\n");
+dbg_xfrin_exec_verb(
+		char *name = knot_dname_to_str(
+			knot_rrset_owner(chset->add[i]));
+		dbg_xfrin_verb("Adding RRSet: %s, type: %s\n", name,
+                               knot_rrtype_to_string(
+		                 knot_rrset_type(chset->add[i])));
+		free(name);
+);
+dbg_xfrin_exec_detail(
 		knot_rrset_dump(chset->add[i], 0);
+);
 
 		is_nsec3 = 0;
 
@@ -2555,7 +2624,7 @@ static int xfrin_apply_add2(knot_zone_contents_t *contents,
 			if (node == NULL) {
 				// create new node, connect it properly to the
 				// zone nodes
-				dbg_xfrin("Node not found. Creating new.\n");
+				dbg_xfrin_verb("Node not found. Creating new.\n");
 				node = xfrin_add_new_node(contents,
 				                          chset->add[i],
 				                          is_nsec3);
@@ -2579,8 +2648,8 @@ static int xfrin_apply_add2(knot_zone_contents_t *contents,
 
 		assert(ret != KNOT_EOK);
 
-		dbg_xfrin("xfrin_apply_..() returned %d, rrset: %p\n", ret,
-		          rrset);
+		dbg_xfrin_verb("xfrin_apply_..() returned %d, rrset: %p\n", ret,
+		               rrset);
 
 		if (ret > 0) {
 			if (ret == 1) {
@@ -2647,6 +2716,7 @@ static int xfrin_apply_replace_soa2(knot_zone_contents_t *contents,
                                     knot_changes_t *changes,
                                     knot_changeset_t *chset)
 {
+	dbg_xfrin("Replacing SOA record.\n");
 	knot_node_t *node = knot_zone_contents_get_apex(contents);
 	assert(node != NULL);
 
@@ -2658,6 +2728,7 @@ static int xfrin_apply_replace_soa2(knot_zone_contents_t *contents,
 	contents->apex = node;
 
 	// remove the SOA RRSet from the apex
+	dbg_xfrin_verb("Removing SOA.\n");
 	knot_rrset_t *rrset = knot_node_remove_rrset(node, KNOT_RRTYPE_SOA);
 	assert(rrset != NULL);
 
@@ -2704,6 +2775,7 @@ static int xfrin_apply_replace_soa2(knot_zone_contents_t *contents,
 	changes->old_rrsets[changes->old_rrsets_count++] = rrset;
 
 	/*! \todo Maybe check if the SOA does not have more RDATA? */
+	dbg_xfrin_verb("Adding RDATA from old SOA to the list of old RDATA.\n");
 	xfrin_changes_add_rdata(changes->old_rdata, changes->old_rdata_types,
 	                        &changes->old_rdata_count, rrset->rdata,
 	                        KNOT_RRTYPE_SOA);
@@ -2725,6 +2797,7 @@ static int xfrin_apply_replace_soa2(knot_zone_contents_t *contents,
 
 	changes->new_rrsets[changes->new_rrsets_count++] = chset->soa_to;
 
+	dbg_xfrin_verb("Adding RDATA from new SOA to the list of new RDATA.\n");
 	xfrin_changes_add_rdata(changes->new_rdata,
 	                        changes->new_rdata_types,
 	                        &changes->new_rdata_count,
@@ -2750,6 +2823,9 @@ static int xfrin_apply_changeset2(knot_zone_contents_t *contents,
 	 * SOA in the zone apex.
 	 */
 
+	dbg_xfrin("APPLYING CHANGESET: from serial %u to serial %u\n",
+	          chset->serial_from, chset->serial_to);
+
 	// check if serial matches
 	/*! \todo Only if SOA is present? */
 	const knot_rrset_t *soa = knot_node_rrset(contents->apex,
@@ -2846,8 +2922,10 @@ static int xfrin_remove_empty_nodes(knot_zone_contents_t *contents,
 {
 	int ret;
 
-	dbg_xfrin("OLD NODES COUNT: %d\n", changes->old_nodes_count);
-	dbg_xfrin("OLD NSEC3 NODES COUNT: %d\n", changes->old_nsec3_count);
+	dbg_xfrin("Removing empty nodes from zone.\n");
+
+	dbg_xfrin_verb("OLD NODES COUNT: %d\n", changes->old_nodes_count);
+	dbg_xfrin_verb("OLD NSEC3 NODES COUNT: %d\n", changes->old_nsec3_count);
 
 	// walk through the zone and select nodes to be removed
 	ret = knot_zone_contents_tree_apply_inorder_reverse(contents,
@@ -2861,8 +2939,8 @@ static int xfrin_remove_empty_nodes(knot_zone_contents_t *contents,
 	                                                 (void *)changes);
 	assert(ret == KNOT_EOK);
 
-	dbg_xfrin("OLD NODES COUNT: %d\n", changes->old_nodes_count);
-	dbg_xfrin("OLD NSEC3 NODES COUNT: %d\n", changes->old_nsec3_count);
+	dbg_xfrin_verb("OLD NODES COUNT: %d\n", changes->old_nodes_count);
+	dbg_xfrin_verb("OLD NSEC3 NODES COUNT: %d\n", changes->old_nsec3_count);
 
 	// remove these nodes from both hash table and the tree
 	ck_hash_table_item_t *hash_item = NULL;
@@ -2921,73 +2999,6 @@ static int xfrin_remove_empty_nodes(knot_zone_contents_t *contents,
 
 /*----------------------------------------------------------------------------*/
 
-//static int xfrin_adjust_contents(knot_zone_contents_t *contents,
-//                                 xfrin_changes_t *changes)
-//{
-//	// first, load the NSEC3PARAM record, if there is one
-//	const knot_node_t *apex = knot_zone_contents_apex(contents);
-//	assert(apex != NULL);
-//	const knot_rrset_t *rrset = knot_node_rrset(apex,
-//	                                            KNOT_RRTYPE_NSEC3PARAM);
-
-//	if (rrset != NULL) {
-//		knot_nsec3_params_from_wire(&contents->nsec3_params, rrset);
-//	} else {
-//		memset(&contents->nsec3_params, 0, sizeof(knot_nsec3_params_t));
-//	}
-
-//	// ^ OK
-
-//	xfrin_adjust_data_t data;
-////	data.changes = changes;
-//	data.contents = contents;
-//	data.err = KNOT_EOK;
-
-//	/*
-//	 * Select and delete empty nodes.
-//	 *
-//	 * TODO: do separately
-//	 */
-//	int ret = xfrin_remove_empty_nodes(contents, changes);
-//	if (ret != KNOT_EOK) {
-//		dbg_xfrin("Failed to remove empty nodes: %s\n",
-//		          knot_strerror(ret));
-//		return ret;
-//	}
-
-//	/*
-//	 * Walk through the nodes in the zone and do the following:
-//	 * 1) Check node flags (authoritative / non-authoritative / deleg. pt.).
-//	 * 2) For each node without NSEC3 node, try to find the NSEC3 node.
-//	 * 3) Insert all dnames to dname table.
-//	 *
-//	 * We must also set previous nodes, as this cannot be done during
-//	 * node insertion - in that time we do not know, if the node is
-//	 * authoritative or no.
-//	 */
-
-//	knot_zone_contents_tree_apply_inorder(contents,
-//	                                      xfrin_adjust_node_in_tree,
-//	                                      (void *)&data);
-
-//	if (data.err != KNOT_EOK) {
-//		dbg_xfrin("Failed to adjust nodes after IXFR: %s\n",
-//		          knot_strerror(data.err));
-//		return data.err;
-//	}
-
-//	/*
-//	 * In second walkthrough check CNAME loops, including wildcards.
-//	 */
-//	knot_zone_contents_tree_apply_inorder(contents,
-//	                                      xfrin_check_loops_in_tree,
-//	                                      (void *)&data);
-
-//	return data.err;
-//}
-
-/*----------------------------------------------------------------------------*/
-
 static void xfrin_check_contents_copy_node(knot_node_t *node, void *data)
 {
 	assert(node != NULL);
