@@ -243,9 +243,72 @@ static knot_rrtype_descriptor_t
   	  { KNOT_RDATA_WF_BINARY },
 	  { KNOT_RDATA_ZF_UNKNOWN }, true },
   	/* 42 */
-	{ KNOT_RRTYPE_APL, "APL", 1,
-  	  { KNOT_RDATA_WF_APL },
-    	  { KNOT_RDATA_ZF_APL }, false },
+	{ KNOT_RRTYPE_APL, "APL", 64,
+	         { KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL,
+	           KNOT_RDATA_WF_APL, KNOT_RDATA_WF_APL },
+          { KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL,
+            KNOT_RDATA_ZF_APL, KNOT_RDATA_ZF_APL }, 
+          false },
   	/* 43 */
   	{ KNOT_RRTYPE_DS, "DS", 4,
   	  { KNOT_RDATA_WF_SHORT, KNOT_RDATA_WF_BYTE,
@@ -310,6 +373,13 @@ static knot_rrtype_descriptor_t
 	  { KNOT_RDATA_ZF_BYTE, KNOT_RDATA_ZF_BYTE,
 	    KNOT_RDATA_ZF_SHORT, KNOT_RDATA_ZF_HEX_LEN }, true },
   	/* 52 */
+  	{ KNOT_RRTYPE_TLSA, "TLSA", 4,
+  	  { KNOT_RDATA_WF_BYTE,
+  	    KNOT_RDATA_WF_BYTE,
+  	    KNOT_RDATA_WF_BYTE,
+  	    KNOT_RDATA_WF_BINARY },
+	  { KNOT_RDATA_ZF_BYTE, KNOT_RDATA_ZF_BYTE,
+	    KNOT_RDATA_ZF_BYTE, KNOT_RDATA_ZF_HEX }, true },
 
 
     /* In NSD they have indices between 52 and 99 filled with
