@@ -48,6 +48,8 @@ enum knot_error {
 	KNOT_EFEWDATA,        /*!< Not enough data to parse. */
 	KNOT_ESPACE,          /*!< Not enough space provided. */
 	KNOT_EMALF,           /*!< Malformed data. */
+	KNOT_ENOENT,          /*!< Resource not found. */
+	KNOT_EACCES,          /*!< Permission is denied. */ 
 	KNOT_ECRYPTO,         /*!< Error in crypto library. */
 	KNOT_ENSEC3PAR,       /*!< Missing or wrong NSEC3PARAM record. */
 	KNOT_EBADZONE,        /*!< Domain name does not belong to the zone. */
@@ -65,7 +67,8 @@ enum knot_error {
 	KNOT_EXFRREFUSED,     /*!< Zone transfer refused by the server. */
 	KNOT_ECONN,           /*!< Connection reset. */
 	KNOT_EIXFRSPACE,      /*!< IXFR reply did not fit in. */
-	KNOT_ERROR_COUNT = 31
+	KNOT_ECNAME,          /*!< CNAME loop found in zone. */
+	KNOT_ERROR_COUNT = 34
 };
 
 /*! \brief Table linking error messages to error codes. */
