@@ -15,8 +15,6 @@
  */
 
 #include "util/error.h"
-#include "util/utils.h"
-
 #include "common/errors.h"
 
 const error_table_t knot_error_msgs[KNOT_ERROR_COUNT] = {
@@ -30,6 +28,8 @@ const error_table_t knot_error_msgs[KNOT_ERROR_COUNT] = {
 	{KNOT_EFEWDATA, "Not enough data to parse."},
 	{KNOT_ESPACE, "Not enough space provided."},
 	{KNOT_EMALF, "Malformed data."},
+	{KNOT_ENOENT, "Resource not found."},
+	{KNOT_EACCES,  "Permission to perform requested operation is denied."},
 	{KNOT_ECRYPTO, "Error in crypto library."},
 	{KNOT_ENSEC3PAR, "Missing or wrong NSEC3PARAM record."},
 	{KNOT_EBADZONE, "Domain name does not belong to the given zone."},
@@ -50,5 +50,6 @@ const error_table_t knot_error_msgs[KNOT_ERROR_COUNT] = {
 	{KNOT_TSIG_EBADTIME, "TSIG signing time out of range." },
 	{KNOT_ECONN, "Connection reset."},
 	{KNOT_EIXFRSPACE, "IXFR reply did not fit in."},
+	{KNOT_ECNAME, "CNAME loop found in zone."},
 	{KNOT_ERROR, 0}
 };
