@@ -81,67 +81,67 @@ static inline void knot_node_flags_set_nonauth(uint8_t *flags)
 	*flags |= KNOT_NODE_FLAGS_NONAUTH;
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Returns the old node flag
- *
- * \param flags Flags to retrieve the flag from.
- *
- * \return A byte with only the old node flag set if it was set in \a flags.
- */
-static inline uint8_t knot_node_flags_get_old(uint8_t flags)
-{
-	return flags & KNOT_NODE_FLAGS_OLD;
-}
+///*----------------------------------------------------------------------------*/
+///*!
+// * \brief Returns the old node flag
+// *
+// * \param flags Flags to retrieve the flag from.
+// *
+// * \return A byte with only the old node flag set if it was set in \a flags.
+// */
+//static inline uint8_t knot_node_flags_get_old(uint8_t flags)
+//{
+//	return flags & KNOT_NODE_FLAGS_OLD;
+//}
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Sets the old node flag.
- *
- * \param flags Flags to set the flag in.
- */
-static inline void knot_node_flags_set_new(uint8_t *flags)
-{
-	*flags |= KNOT_NODE_FLAGS_NEW;
-}
+///*----------------------------------------------------------------------------*/
+///*!
+// * \brief Sets the old node flag.
+// *
+// * \param flags Flags to set the flag in.
+// */
+//static inline void knot_node_flags_set_new(uint8_t *flags)
+//{
+//	*flags |= KNOT_NODE_FLAGS_NEW;
+//}
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Returns the new node flag
- *
- * \param flags Flags to retrieve the flag from.
- *
- * \return A byte with only the new node flag set if it was set in \a flags.
- */
-static inline uint8_t knot_node_flags_get_new(uint8_t flags)
-{
-	return flags & KNOT_NODE_FLAGS_NEW;
-}
+///*----------------------------------------------------------------------------*/
+///*!
+// * \brief Returns the new node flag
+// *
+// * \param flags Flags to retrieve the flag from.
+// *
+// * \return A byte with only the new node flag set if it was set in \a flags.
+// */
+//static inline uint8_t knot_node_flags_get_new(uint8_t flags)
+//{
+//	return flags & KNOT_NODE_FLAGS_NEW;
+//}
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Sets the new node flag.
- *
- * \param flags Flags to set the flag in.
- */
-static inline void knot_node_flags_set_old(uint8_t *flags)
-{
-	*flags |= KNOT_NODE_FLAGS_OLD;
-}
+///*----------------------------------------------------------------------------*/
+///*!
+// * \brief Sets the new node flag.
+// *
+// * \param flags Flags to set the flag in.
+// */
+//static inline void knot_node_flags_set_old(uint8_t *flags)
+//{
+//	*flags |= KNOT_NODE_FLAGS_OLD;
+//}
 
-/*----------------------------------------------------------------------------*/
+///*----------------------------------------------------------------------------*/
 
-static inline void knot_node_flags_clear_new(uint8_t *flags)
-{
-	*flags &= ~KNOT_NODE_FLAGS_NEW;
-}
+//static inline void knot_node_flags_clear_new(uint8_t *flags)
+//{
+//	*flags &= ~KNOT_NODE_FLAGS_NEW;
+//}
 
-/*----------------------------------------------------------------------------*/
+///*----------------------------------------------------------------------------*/
 
-static inline void knot_node_flags_clear_old(uint8_t *flags)
-{
-	*flags &= ~KNOT_NODE_FLAGS_OLD;
-}
+//static inline void knot_node_flags_clear_old(uint8_t *flags)
+//{
+//	*flags &= ~KNOT_NODE_FLAGS_OLD;
+//}
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -167,30 +167,30 @@ static int compare_rrset_types(void *rr1, void *rr2)
 
 /*----------------------------------------------------------------------------*/
 
-static int knot_node_zone_gen_is_new(const knot_node_t *node)
-{
-	assert(node->zone != NULL);
-	knot_zone_contents_t *cont = rcu_dereference(node->zone->contents);
-	assert(cont != NULL);
-	return knot_zone_contents_gen_is_new(cont);
-}
+//static int knot_node_zone_gen_is_new(const knot_node_t *node)
+//{
+//	assert(node->zone != NULL);
+//	knot_zone_contents_t *cont = rcu_dereference(node->zone->contents);
+//	assert(cont != NULL);
+//	return knot_zone_contents_gen_is_new(cont);
+//}
 
-/*----------------------------------------------------------------------------*/
+///*----------------------------------------------------------------------------*/
 
-static int knot_node_zone_gen_is_old(const knot_node_t *node)
-{
-	assert(node->zone != NULL);
-	knot_zone_contents_t *cont = rcu_dereference(node->zone->contents);
-	assert(cont != NULL);
-	return knot_zone_contents_gen_is_old(cont);
-}
+//static int knot_node_zone_gen_is_old(const knot_node_t *node)
+//{
+//	assert(node->zone != NULL);
+//	knot_zone_contents_t *cont = rcu_dereference(node->zone->contents);
+//	assert(cont != NULL);
+//	return knot_zone_contents_gen_is_old(cont);
+//}
 
 /*----------------------------------------------------------------------------*/
 /* API functions                                                              */
 /*----------------------------------------------------------------------------*/
 
 knot_node_t *knot_node_new(knot_dname_t *owner, knot_node_t *parent,
-                               uint8_t flags)
+                           uint8_t flags)
 {
 	knot_node_t *ret = (knot_node_t *)calloc(1, sizeof(knot_node_t));
 	if (ret == NULL) {
@@ -215,6 +215,10 @@ knot_node_t *knot_node_new(knot_dname_t *owner, knot_node_t *parent,
 int knot_node_add_rrset(knot_node_t *node, knot_rrset_t *rrset,
                           int merge)
 {
+	if (node == NULL) {
+		return KNOT_EBADARG;
+	}
+
 	int ret = 0;
 
 	if ((ret = (gen_tree_add(node->rrset_tree, rrset,
@@ -236,7 +240,10 @@ int knot_node_add_rrset(knot_node_t *node, knot_rrset_t *rrset,
 const knot_rrset_t *knot_node_rrset(const knot_node_t *node,
                                         uint16_t type)
 {
-	assert(node != NULL);
+	if (node == NULL) {
+		return NULL;
+	}
+
 	assert(node->rrset_tree != NULL);
 	knot_rrset_t rrset;
 	rrset.type = type;
@@ -247,6 +254,10 @@ const knot_rrset_t *knot_node_rrset(const knot_node_t *node,
 
 knot_rrset_t *knot_node_get_rrset(const knot_node_t *node, uint16_t type)
 {
+	if (node == NULL) {
+		return NULL;
+	}
+
 	knot_rrset_t rrset;
 	rrset.type = type;
 	return (knot_rrset_t *)gen_tree_find(node->rrset_tree, &rrset);
@@ -256,6 +267,10 @@ knot_rrset_t *knot_node_get_rrset(const knot_node_t *node, uint16_t type)
 
 knot_rrset_t *knot_node_remove_rrset(knot_node_t *node, uint16_t type)
 {
+	if (node == NULL) {
+		return NULL;
+	}
+
 	knot_rrset_t dummy_rrset;
 	dummy_rrset.type = type;
 	knot_rrset_t *rrset =
@@ -271,6 +286,10 @@ knot_rrset_t *knot_node_remove_rrset(knot_node_t *node, uint16_t type)
 
 void knot_node_remove_all_rrsets(knot_node_t *node)
 {
+	if (node == NULL) {
+		return;
+	}
+
 	// remove RRSets but do not delete them
 	gen_tree_clear(node->rrset_tree);
 	node->rrset_count = 0;
@@ -281,6 +300,10 @@ void knot_node_remove_all_rrsets(knot_node_t *node)
 
 short knot_node_rrset_count(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return KNOT_EBADARG;
+	}
+
 	return node->rrset_count;
 }
 
@@ -289,20 +312,26 @@ short knot_node_rrset_count(const knot_node_t *node)
 struct knot_node_save_rrset_arg {
 	knot_rrset_t **array;
 	size_t count;
+	size_t max_count;
 };
 
 static void save_rrset_to_array(void *node, void *data)
 {
-	knot_rrset_t *rrset = (knot_rrset_t *)node;
 	struct knot_node_save_rrset_arg *args =
 		(struct knot_node_save_rrset_arg *)data;
+	knot_rrset_t *rrset = (knot_rrset_t *)node;
+
+	if (args->count > args->max_count) {
+		++args->count;
+		return;
+	}
+
 	args->array[args->count++] = rrset;
 }
 
 knot_rrset_t **knot_node_get_rrsets(const knot_node_t *node)
 {
-//	knot_node_dump(node, 1);
-	if (node->rrset_count == 0) {
+	if (node == NULL || node->rrset_count == 0) {
 		return NULL;
 	}
 	knot_rrset_t **rrsets = (knot_rrset_t **)malloc(
@@ -311,9 +340,9 @@ knot_rrset_t **knot_node_get_rrsets(const knot_node_t *node)
 	struct knot_node_save_rrset_arg args;
 	args.array = rrsets;
 	args.count = 0;
+	args.max_count = node->rrset_count;
 
-	gen_tree_apply_inorder(node->rrset_tree, save_rrset_to_array,
-	                       &args);
+	gen_tree_apply_inorder(node->rrset_tree, save_rrset_to_array, &args);
 
 	assert(args.count == node->rrset_count);
 
@@ -324,46 +353,71 @@ knot_rrset_t **knot_node_get_rrsets(const knot_node_t *node)
 
 const knot_rrset_t **knot_node_rrsets(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return NULL;
+	}
+
 	return (const knot_rrset_t **)knot_node_get_rrsets(node);
 }
 
 /*----------------------------------------------------------------------------*/
 
-const knot_node_t *knot_node_parent(const knot_node_t *node, 
-                                        int check_version)
+static void count_rrsets(void *node, void *data)
 {
-//	assert(!check_version
-//	       || (node->zone != NULL && node->zone->contents != NULL));
-	
-	knot_node_t *parent = node->parent;
-	
-	if (check_version && node->zone != NULL) {
-		int new_gen = knot_node_zone_gen_is_new(node);
-//		short ver = knot_node_zone_generation(node);
-	
-		/*! \todo Remove, this will not be true during the reference
-		 *        fixing.
-		 */
-//		assert(new_gen || parent == NULL
-//		       || !knot_node_is_new(parent));
+	assert(node != NULL);
+	assert(data != NULL);
 
-		if (new_gen && parent != NULL) {
-			// we want the new node
-			assert(node->parent->new_node != NULL);
-			parent = parent->new_node;
-		}
+	int *count = (int *)data;
+	++(*count);
+}
+
+/*----------------------------------------------------------------------------*/
+
+int knot_node_count_rrsets(const knot_node_t *node)
+{
+	if (node == NULL) {
+		return KNOT_EBADARG;
 	}
-	
-	return parent;
+
+	int count = 0;
+	gen_tree_apply_inorder(node->rrset_tree, count_rrsets, (void *)&count);
+
+	return count;
+}
+
+/*----------------------------------------------------------------------------*/
+
+const knot_node_t *knot_node_parent(const knot_node_t *node)
+{
+	if (node == NULL) {
+		return NULL;
+	}
+
+	return knot_node_get_parent(node);
+}
+
+/*----------------------------------------------------------------------------*/
+
+knot_node_t *knot_node_get_parent(const knot_node_t *node)
+{
+	if (node == NULL) {
+		return NULL;
+	}
+
+	return node->parent;
 }
 
 /*----------------------------------------------------------------------------*/
 
 void knot_node_set_parent(knot_node_t *node, knot_node_t *parent)
 {
+	if (node == NULL || node->parent == parent) {
+		return;
+	}
+
 	// decrease number of children of previous parent
 	if (node->parent != NULL) {
-		--parent->children;
+		--node->parent->children;
 	}
 	// set the parent
 	node->parent = parent;
@@ -378,105 +432,87 @@ void knot_node_set_parent(knot_node_t *node, knot_node_t *parent)
 
 unsigned int knot_node_children(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return KNOT_EBADARG;
+	}
+
 	return node->children;
 }
 
 /*----------------------------------------------------------------------------*/
 
-const knot_node_t *knot_node_previous(const knot_node_t *node, 
-                                          int check_version)
+const knot_node_t *knot_node_previous(const knot_node_t *node)
 {
-	return knot_node_get_previous(node, check_version);
+	if (node == NULL) {
+		return NULL;
+	}
+
+	return knot_node_get_previous(node);
 }
 
 /*----------------------------------------------------------------------------*/
 
-knot_node_t *knot_node_get_previous(const knot_node_t *node, 
-                                        int check_version)
+knot_node_t *knot_node_get_previous(const knot_node_t *node)
 {
-	assert(!check_version
-	       || (node->zone != NULL && node->zone->contents != NULL));
-	
-	knot_node_t *prev = node->prev;
-	
-	if (check_version && prev != NULL) {
-		int new_gen = knot_node_zone_gen_is_new(node);
-		int old_gen = knot_node_zone_gen_is_old(node);
-//		short ver = knot_node_zone_generation(node);
-		
-		if (old_gen) {  // we want old node
-			while (knot_node_is_new(prev)) {
-				prev = prev->prev;
-			}
-			assert(!knot_node_is_new(prev));
-		} else if (new_gen) {  // we want new node
-			while (knot_node_is_old(prev)) {
-				if (prev->new_node) {
-					prev = prev->new_node;
-				} else {
-					prev = prev;
-				}
-			}
-			assert(knot_node_is_new(prev));
-		}
+	if (node == NULL) {
+		return NULL;
 	}
 	
-	return prev;
+	return node->prev;
+}
+
+/*----------------------------------------------------------------------------*/
+
+const knot_node_t *knot_node_next(const knot_node_t *node)
+{
+	if (node == NULL) {
+		return NULL;
+	}
+
+	return node->next;
 }
 
 /*----------------------------------------------------------------------------*/
 
 void knot_node_set_previous(knot_node_t *node, knot_node_t *prev)
 {
-	node->prev = prev;
-	if (prev != NULL) {
-		// set the prev pointer of the next node to the given node
-		if (prev->next != NULL) {
-			assert(prev->next->prev == prev);
-			prev->next->prev = node;
-		}
-		node->next = prev->next;
-		prev->next = node;
+	if (node == NULL) {
+		return;
 	}
+
+	node->prev = prev;
 }
 
 /*----------------------------------------------------------------------------*/
 
-knot_node_t *knot_node_get_nsec3_node(const knot_node_t *node,
-                                      int check_version)
+knot_node_t *knot_node_get_nsec3_node(const knot_node_t *node)
 {
-	knot_node_t *nsec3_node = node->nsec3_node;
-	if (nsec3_node == NULL) {
+	if (node == NULL) {
 		return NULL;
 	}
 	
-	if (check_version) {
-		int new_gen = knot_node_zone_gen_is_new(node);
-		int old_gen = knot_node_zone_gen_is_old(node);
-//		short ver = knot_node_zone_generation(node);
-		assert(new_gen || !knot_node_is_new(nsec3_node));
-		if (old_gen && knot_node_is_new(nsec3_node)) {
-			return NULL;
-		} else if (new_gen && knot_node_is_old(nsec3_node)) {
-			nsec3_node = nsec3_node->new_node;
-		}
-	}
-	
-	return nsec3_node;
+	return node->nsec3_node;
 }
 
 /*----------------------------------------------------------------------------*/
 
-const knot_node_t *knot_node_nsec3_node(const knot_node_t *node,
-                                        int check_version)
+const knot_node_t *knot_node_nsec3_node(const knot_node_t *node)
 {
-	return knot_node_get_nsec3_node(node, check_version);
+	if (node == NULL) {
+		return NULL;
+	}
+
+	return knot_node_get_nsec3_node(node);
 }
 
 /*----------------------------------------------------------------------------*/
 
 void knot_node_set_nsec3_node(knot_node_t *node, knot_node_t *nsec3_node)
 {
+	if (node == NULL) {
+		return;
+	}
+
 	node->nsec3_node = nsec3_node;
 	if (nsec3_node != NULL) {
 		nsec3_node->nsec3_referer = node;
@@ -487,6 +523,10 @@ void knot_node_set_nsec3_node(knot_node_t *node, knot_node_t *nsec3_node)
 
 const knot_dname_t *knot_node_owner(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return NULL;
+	}
+
 	return node->owner;
 }
 
@@ -511,89 +551,47 @@ void knot_node_set_owner(knot_node_t *node, knot_dname_t* owner)
 
 /*----------------------------------------------------------------------------*/
 
-const knot_node_t *knot_node_wildcard_child(const knot_node_t *node, 
-                                                int check_version)
+knot_node_t *knot_node_get_wildcard_child(const knot_node_t *node)
 {
-	knot_node_t *w = node->wildcard_child;
-	
-	if (check_version && w != 0) {
-		int new_gen = knot_node_zone_gen_is_new(node);
-		int old_gen = knot_node_zone_gen_is_old(node);
-//		short ver = knot_node_zone_generation(node);
-
-		if (old_gen && knot_node_is_new(w)) {
-			return NULL;
-		} else if (new_gen && knot_node_is_old(w)) {
-			assert(w->new_node != NULL);
-			w = w->new_node;
-		}
+	if (node == NULL) {
+		return NULL;
 	}
-	
-	return w;
+
+	return node->wildcard_child;
 }
 
 /*----------------------------------------------------------------------------*/
 
 void knot_node_set_wildcard_child(knot_node_t *node,
-                                    knot_node_t *wildcard_child)
+                                  knot_node_t *wildcard_child)
 {
+	if (node == NULL) {
+		return;
+	}
+
 	node->wildcard_child = wildcard_child;
 //	assert(wildcard_child->parent == node);
 }
 
 /*----------------------------------------------------------------------------*/
 
-const knot_node_t *knot_node_current(const knot_node_t *node)
+const knot_node_t *knot_node_wildcard_child(const knot_node_t *node)
 {
-	if (node == NULL || node->zone == NULL
-	    || knot_zone_contents(node->zone) == NULL) {
-		return node;
-	}
-
-	int new_gen = knot_node_zone_gen_is_new(node);
-	int old_gen = knot_node_zone_gen_is_old(node);
-//	short ver = knot_node_zone_generation(node);
-
-	if (old_gen && knot_node_is_new(node)) {
+	if (node == NULL) {
 		return NULL;
-	} else if (new_gen && knot_node_is_old(node)) {
-		assert(node->new_node != NULL);
-		return node->new_node;
-	}
-	return node;
-}
-
-/*----------------------------------------------------------------------------*/
-
-knot_node_t *knot_node_get_current(knot_node_t *node)
-{
-	if (node == NULL || node->zone == NULL
-	    || knot_zone_contents(node->zone) == NULL) {
-		return node;
 	}
 
-	int new_gen = knot_node_zone_gen_is_new(node);
-	int old_gen = knot_node_zone_gen_is_old(node);
-//	short ver = knot_node_zone_generation(node);
-
-	if (old_gen && knot_node_is_new(node)) {
-		return NULL;
-	} else if (new_gen && knot_node_is_old(node)) {
-		assert(node->new_node != NULL);
-		return node->new_node;
-	}
-	
-	assert((old_gen && knot_node_is_old(node))
-	       || (new_gen && knot_node_is_new(node))
-	       || (!old_gen && !new_gen));
-	
-	return node;
+	return knot_node_get_wildcard_child(node);
 }
 
 /*----------------------------------------------------------------------------*/
 
 const knot_node_t *knot_node_new_node(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return NULL;
+	}
+
 	return node->new_node;
 }
 
@@ -601,6 +599,10 @@ const knot_node_t *knot_node_new_node(const knot_node_t *node)
 
 knot_node_t *knot_node_get_new_node(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return NULL;
+	}
+
 	return node->new_node;
 }
 
@@ -609,6 +611,10 @@ knot_node_t *knot_node_get_new_node(const knot_node_t *node)
 void knot_node_set_new_node(knot_node_t *node,
                               knot_node_t *new_node)
 {
+	if (node == NULL) {
+		return;
+	}
+
 	node->new_node = new_node;
 }
 
@@ -616,6 +622,10 @@ void knot_node_set_new_node(knot_node_t *node,
 
 void knot_node_set_zone(knot_node_t *node, knot_zone_t *zone)
 {
+	if (node == NULL) {
+		return;
+	}
+
 	node->zone = zone;
 }
 
@@ -623,7 +633,7 @@ void knot_node_set_zone(knot_node_t *node, knot_zone_t *zone)
 
 void knot_node_update_ref(knot_node_t **ref)
 {
-	if (*ref != NULL && knot_node_is_old(*ref)) {
+	if (*ref != NULL && (*ref)->new_node != NULL) {
 		*ref = (*ref)->new_node;
 	}
 }
@@ -632,58 +642,28 @@ void knot_node_update_ref(knot_node_t **ref)
 
 void knot_node_update_refs(knot_node_t *node)
 {
-	/* CLEANUP */
-	/* OMG! */
 	// reference to previous node
 	knot_node_update_ref(&node->prev);
-//	if (node->prev && knot_node_is_old(node->prev)) {
-//		assert(node->prev->new_node != NULL);
-//		node->prev = node->prev->new_node;
-//	}
-
 	// reference to next node
 	knot_node_update_ref(&node->next);
-//	if (node->next && knot_node_is_old(node->next)) {
-//		assert(node->next->new_node != NULL);
-//		node->next = node->next->new_node;
-//	}
-
 	// reference to parent
-//	if (node->parent && knot_node_is_old(node->parent)) {
-//		assert(node->parent->new_node != NULL);
-//		// do not use the API function to set parent, so that children count
-//		// is not changed
-//		//knot_node_set_parent(node, node->parent->new_node);
-//		node->parent = node->parent->new_node;
-//	}
 	knot_node_update_ref(&node->parent);
-
 	// reference to wildcard child
 	knot_node_update_ref(&node->wildcard_child);
-//	if (node->wildcard_child && knot_node_is_old(node->wildcard_child)) {
-//		assert(node->wildcard_child->new_node != NULL);
-//		node->wildcard_child = node->wildcard_child->new_node;
-//	}
-
 	// reference to NSEC3 node
 	knot_node_update_ref(&node->nsec3_node);
-//	if (node->nsec3_node && knot_node_is_old(node->nsec3_node)) {
-//		assert(node->nsec3_node->new_node != NULL);
-//		node->nsec3_node = node->nsec3_node->new_node;
-//	}
-
 	// reference to NSEC3 referrer
 	knot_node_update_ref(&node->nsec3_referer);
-//	if (node->nsec3_referer && knot_node_is_old(node->nsec3_referer)) {
-//		assert(node->nsec3_referer->new_node != NULL);
-//		node->nsec3_referer = node->nsec3_referer->new_node;
-//	}
 }
 
 /*----------------------------------------------------------------------------*/
 
 void knot_node_set_deleg_point(knot_node_t *node)
 {
+	if (node == NULL) {
+		return;
+	}
+
 	knot_node_flags_set_deleg(&node->flags);
 }
 
@@ -691,6 +671,10 @@ void knot_node_set_deleg_point(knot_node_t *node)
 
 int knot_node_is_deleg_point(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return KNOT_EBADARG;
+	}
+
 	return knot_node_flags_get_deleg(node->flags);
 }
 
@@ -698,6 +682,10 @@ int knot_node_is_deleg_point(const knot_node_t *node)
 
 void knot_node_set_non_auth(knot_node_t *node)
 {
+	if (node == NULL) {
+		return;
+	}
+
 	knot_node_flags_set_nonauth(&node->flags);
 }
 
@@ -705,6 +693,10 @@ void knot_node_set_non_auth(knot_node_t *node)
 
 int knot_node_is_non_auth(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return KNOT_EBADARG;
+	}
+
 	return knot_node_flags_get_nonauth(node->flags);
 }
 
@@ -712,53 +704,15 @@ int knot_node_is_non_auth(const knot_node_t *node)
 
 int knot_node_is_auth(const knot_node_t *node)
 {
+	if (node == NULL) {
+		return KNOT_EBADARG;
+	}
+
 	return (node->flags == 0);
 }
 
 /*----------------------------------------------------------------------------*/
 
-int knot_node_is_new(const knot_node_t *node)
-{
-	return knot_node_flags_get_new(node->flags);
-}
-
-/*----------------------------------------------------------------------------*/
-
-int knot_node_is_old(const knot_node_t *node)
-{
-	return knot_node_flags_get_old(node->flags);
-}
-
-/*----------------------------------------------------------------------------*/
-
-void knot_node_set_new(knot_node_t *node)
-{
-	knot_node_flags_set_new(&node->flags);
-}
-
-/*----------------------------------------------------------------------------*/
-
-void knot_node_set_old(knot_node_t *node)
-{
-	knot_node_flags_set_old(&node->flags);
-}
-
-/*----------------------------------------------------------------------------*/
-
-void knot_node_clear_new(knot_node_t *node)
-{
-	knot_node_flags_clear_new(&node->flags);
-}
-
-/*----------------------------------------------------------------------------*/
-
-void knot_node_clear_old(knot_node_t *node)
-{
-	knot_node_flags_clear_old(&node->flags);
-}
-
-/*----------------------------------------------------------------------------*/
-
 static void knot_node_free_rrsets_from_tree(void *item, void *data)
 {
 	if (item == NULL) {
@@ -781,6 +735,10 @@ void knot_node_free_rrsets(knot_node_t *node, int free_rdata_dnames)
 	
 //	free(rrsets);
 
+	if (node == NULL) {
+		return;
+	}
+
 	char *name = knot_dname_to_str(node->owner);
 	free(name);
 
@@ -790,23 +748,28 @@ void knot_node_free_rrsets(knot_node_t *node, int free_rdata_dnames)
 
 /*----------------------------------------------------------------------------*/
 
-void knot_node_free(knot_node_t **node, int free_owner, int fix_refs)
+void knot_node_free(knot_node_t **node, int fix_refs)
 {
 	if (node == NULL || *node == NULL) {
 		return;
 	}
 	
-	dbg_node("Freeing node.\n");
+	dbg_node("Freeing node: %p\n", *node);
+
 	if ((*node)->rrset_tree != NULL) {
 		dbg_node("Freeing RRSets.\n");
 		gen_tree_destroy(&(*node)->rrset_tree, NULL, NULL);
 	}
 
-	/*! \todo Always release owner? */
-	//if (free_owner) {
-		dbg_node("Releasing owner.\n");
-		knot_dname_release((*node)->owner);
-	//}
+	// set owner's node pointer to NULL, but only if the 'node' does
+	// not point to the owner's node
+	if (node != &(*node)->owner->node
+	    && knot_dname_node(knot_node_owner(*node)) == *node) {
+		knot_dname_set_node((*node)->owner, NULL);
+	}
+
+	dbg_node("Releasing owner.\n");
+	knot_dname_release((*node)->owner);
 
 	// check nodes referencing this node and fix the references
 
@@ -858,6 +821,8 @@ void knot_node_free(knot_node_t **node, int free_owner, int fix_refs)
 
 int knot_node_compare(knot_node_t *node1, knot_node_t *node2)
 {
+	assert(node1 != NULL && node2 != NULL);
+
 	return knot_dname_compare(node1->owner, node2->owner);
 }
 
@@ -865,12 +830,19 @@ int knot_node_compare(knot_node_t *node1, knot_node_t *node2)
 
 int knot_node_shallow_copy(const knot_node_t *from, knot_node_t **to)
 {
+	if (from == NULL || to == NULL) {
+		return KNOT_EBADARG;
+	}
+
 	// create new node
-	*to = knot_node_new(from->owner, from->parent, from->flags);
+	*to = knot_node_new(from->owner, NULL, from->flags);
 	if (*to == NULL) {
 		return KNOT_ENOMEM;
 	}
 
+	// set the parent by hand, so that the children count is not affected
+	(*to)->parent = from->parent;
+
 	/* Free old rrset_tree, as it will be replaced by shallow copy. */
 	gen_tree_destroy(&(*to)->rrset_tree, 0, 0);
 
@@ -880,11 +852,8 @@ int knot_node_shallow_copy(const knot_node_t *from, knot_node_t **to)
 	memcpy(*to, from, sizeof(knot_node_t));
 
 	// copy RRSets
-	// copy the skip list with the old references
-	/* CLEANUP */
 	(*to)->rrset_tree = gen_tree_shallow_copy(from->rrset_tree);
-//	assert((*to)->rrset_tree != from->rrset_tree);
-//	(*to)->rrsets = skip_copy_list(from->rrsets);
+
 	if ((*to)->rrset_tree == NULL) {
 		free(*to);
 		*to = NULL;
