@@ -50,8 +50,6 @@ struct knot_node {
 	/*! \brief Type-ordered list of RRSets belonging to this node. */
 	general_tree_t *rrset_tree;
 
-	unsigned short rrset_count; /*!< Number of RRSets stored in the node. */
-
 	/*! \brief Wildcard node being the direct descendant of this node. */
 	struct knot_node *wildcard_child;
 
@@ -76,6 +74,23 @@ struct knot_node {
 
 	struct knot_node *nsec3_referer;
 
+	struct knot_zone *zone;
+
+	struct knot_node *new_node;
+	
+	unsigned int children;
+
+	unsigned short rrset_count; /*!< Number of RRSets stored in the node. */
+
+	/*!
+	 * \brief Generation of node to be used.
+	 *
+	 * If set to 0, the old node will be used. Otherwise new nodes will
+	 * be used. This applies when getting some referenced node.
+	 
+	 */
+//	short **generation;
+
 	/*!
 	 * \brief Various flags.
 	 *
@@ -86,21 +101,6 @@ struct knot_node {
 	 *   0x40 - node is new, should not be used while zone is old
 	 */
 	uint8_t flags;
-
-	struct knot_node *new_node;
-	
-	unsigned int children;
-	
-	/*!
-	 * \brief Generation of node to be used.
-	 *
-	 * If set to 0, the old node will be used. Otherwise new nodes will
-	 * be used. This applies when getting some referenced node.
-	 
-	 */
-//	short **generation;
-
-	struct knot_zone *zone;
 };
 
 typedef struct knot_node knot_node_t;
@@ -204,6 +204,8 @@ knot_rrset_t **knot_node_get_rrsets(const knot_node_t *node);
  */
 const knot_rrset_t **knot_node_rrsets(const knot_node_t *node);
 
+int knot_node_count_rrsets(const knot_node_t *node);
+
 /*!
  * \brief Returns the parent of the node.
  *
@@ -212,8 +214,9 @@ const knot_rrset_t **knot_node_rrsets(const knot_node_t *node);
  * \return Parent node of the given node or NULL if no parent has been set (e.g.
  *         node in a zone apex has no parent).
  */
-const knot_node_t *knot_node_parent(const knot_node_t *node, 
-                                        int check_version);
+const knot_node_t *knot_node_parent(const knot_node_t *node);
+
+knot_node_t *knot_node_get_parent(const knot_node_t *node);
 
 /*!
  * \brief Sets the parent of the node.
@@ -235,8 +238,7 @@ unsigned int knot_node_children(const knot_node_t *node);
  *         the first node in zone if \a node is the last node in zone.
  * \retval NULL if previous node is not set.
  */
-const knot_node_t *knot_node_previous(const knot_node_t *node, 
-                                          int check_version);
+const knot_node_t *knot_node_previous(const knot_node_t *node);
 
 /*!
  * \brief Returns the previous authoritative node or delegation point in
@@ -251,8 +253,9 @@ const knot_node_t *knot_node_previous(const knot_node_t *node,
  *         the first node in zone if \a node is the last node in zone.
  * \retval NULL if previous node is not set.
  */
-knot_node_t *knot_node_get_previous(const knot_node_t *node, 
-                                        int check_version);
+knot_node_t *knot_node_get_previous(const knot_node_t *node);
+
+const knot_node_t *knot_node_next(const knot_node_t *node);
 
 /*!
  * \brief Sets the previous node of the given node.
@@ -272,8 +275,7 @@ void knot_node_set_previous(knot_node_t *node, knot_node_t *prev);
  *         and the name of the zone \a node belongs to).
  * \retval NULL if the NSEC3 node is not set.
  */
-knot_node_t *knot_node_get_nsec3_node(const knot_node_t *node,
-                                      int check_version);
+knot_node_t *knot_node_get_nsec3_node(const knot_node_t *node);
 
 /*!
  * \brief Returns the NSEC3 node corresponding to the given node.
@@ -285,8 +287,7 @@ knot_node_t *knot_node_get_nsec3_node(const knot_node_t *node,
  *         and the name of the zone \a node belongs to).
  * \retval NULL if the NSEC3 node is not set.
  */
-const knot_node_t *knot_node_nsec3_node(const knot_node_t *node, 
-                                        int check_version);
+const knot_node_t *knot_node_nsec3_node(const knot_node_t *node);
 
 /*!
  * \brief Sets the corresponding NSEC3 node of the given node.
@@ -327,8 +328,7 @@ void knot_node_set_owner(knot_node_t *node, knot_dname_t* owner);
  *
  * \return Wildcard child of the given node or NULL if it has none.
  */
-const knot_node_t *knot_node_wildcard_child(const knot_node_t *node, 
-                                                int check_version);
+const knot_node_t *knot_node_wildcard_child(const knot_node_t *node);
 
 /*!
  * \brief Sets the wildcard child of the node.
@@ -337,18 +337,19 @@ const knot_node_t *knot_node_wildcard_child(const knot_node_t *node,
  * \param wildcard_child Wildcard child of the node.
  */
 void knot_node_set_wildcard_child(knot_node_t *node,
-                                    knot_node_t *wildcard_child);
+                                  knot_node_t *wildcard_child);
 
-const knot_node_t *knot_node_current(const knot_node_t *node);
+knot_node_t *knot_node_get_wildcard_child(const knot_node_t *node);
 
-knot_node_t *knot_node_get_current(knot_node_t *node);
+//const knot_node_t *knot_node_current(const knot_node_t *node);
+
+//knot_node_t *knot_node_get_current(knot_node_t *node);
 
 const knot_node_t *knot_node_new_node(const knot_node_t *node);
 
 knot_node_t *knot_node_get_new_node(const knot_node_t *node);
 
-void knot_node_set_new_node(knot_node_t *node,
-                              knot_node_t *new_node);
+void knot_node_set_new_node(knot_node_t *node, knot_node_t *new_node);
 
 void knot_node_set_zone(knot_node_t *node, struct knot_zone *zone);
 
@@ -427,7 +428,7 @@ void knot_node_free_rrsets(knot_node_t *node, int free_rdata_dnames);
  *
  * \todo Document missing parameters.
  */
-void knot_node_free(knot_node_t **node, int free_owner, int fix_refs);
+void knot_node_free(knot_node_t **node, int fix_refs);
 
 /*!
  * \brief Compares two nodes according to their owner.
