@@ -21,6 +21,7 @@
 #include "zone-tree.h"
 #include "zone/node.h"
 #include "util/error.h"
+#include "util/debug.h"
 
 /*----------------------------------------------------------------------------*/
 /* Non-API functions                                                          */
@@ -87,6 +88,8 @@ static int knot_zone_tree_copy_node(knot_zone_tree_node_t *from,
 	if (ret != KNOT_EOK) {
 		knot_zone_tree_delete_subtree((*to)->avl.avl_left);
 		(*to)->avl.avl_left = NULL;
+		free(*to);
+		*to = NULL;
 		return ret;
 	}
 
@@ -96,23 +99,73 @@ static int knot_zone_tree_copy_node(knot_zone_tree_node_t *from,
 /*----------------------------------------------------------------------------*/
 
 static void knot_zone_tree_free_node(knot_zone_tree_node_t *node,
-                                       int free_data, int free_owner)
+                                       int free_data)
 {
 	if (node == NULL) {
 		return;
 	}
 
-	knot_zone_tree_free_node(node->avl.avl_left, free_data, free_owner);
+	knot_zone_tree_free_node(node->avl.avl_left, free_data);
 
-	knot_zone_tree_free_node(node->avl.avl_right, free_data, free_owner);
+	knot_zone_tree_free_node(node->avl.avl_right, free_data);
 
 	if (free_data) {
-		knot_node_free(&node->node, free_owner, 0);
+		knot_node_free(&node->node, 0);
 	}
 
 	free(node);
 }
 
+/*----------------------------------------------------------------------------*/
+
+static int knot_zone_tree_deep_copy_node(knot_zone_tree_node_t *from,
+                                         knot_zone_tree_node_t **to)
+{
+	if (from == NULL) {
+		*to = NULL;
+		return KNOT_EOK;
+	}
+
+	*to = (knot_zone_tree_node_t *)malloc(sizeof(knot_zone_tree_node_t));
+	if (*to == NULL) {
+		return KNOT_ENOMEM;
+	}
+
+	int ret = knot_node_shallow_copy(from->node, &(*to)->node);
+//	printf("Copied node: %p to node %p. New node1: %p, new node 2: %p\n",
+//	       from->node, (*to)->node, from->node->new_node, (*to)->node->new_node);
+	if (ret != KNOT_EOK) {
+		dbg_zone_verb("Failed to do shallow copy of node.\n");
+		free(*to);
+		return ret;
+	}
+
+	(*to)->avl.avl_height = from->avl.avl_height;
+
+	ret = knot_zone_tree_deep_copy_node(from->avl.avl_left,
+	                                    &(*to)->avl.avl_left);
+	if (ret != KNOT_EOK) {
+		dbg_zone_verb("Failed to do shallow copy of left subtree.\n");
+		return ret;
+	}
+
+	ret = knot_zone_tree_deep_copy_node(from->avl.avl_right,
+	                                    &(*to)->avl.avl_right);
+	if (ret != KNOT_EOK) {
+		dbg_zone_verb("Failed to do shallow copy of right subtree.\n");
+		knot_zone_tree_free_node((*to)->avl.avl_left, 1);
+		(*to)->avl.avl_left = NULL;
+		knot_node_free(&(*to)->node, 0);
+		free(*to);
+		*to = NULL;
+		return ret;
+	}
+
+	knot_node_set_new_node(from->node, (*to)->node);
+
+	return KNOT_EOK;
+}
+
 /*----------------------------------------------------------------------------*/
 /* API functions                                                              */
 /*----------------------------------------------------------------------------*/
@@ -197,7 +250,7 @@ int knot_zone_tree_get(knot_zone_tree_t *tree, const knot_dname_t *owner,
 	knot_zone_tree_node_t *n = TREE_FIND(tree, knot_zone_tree_node, avl,
 	                                       tmp);
 
-	knot_node_free(&tmp_data, 0, 0);
+	knot_node_free(&tmp_data, 0);
 	free(tmp);
 
 	if (n != NULL) {
@@ -212,15 +265,14 @@ int knot_zone_tree_get(knot_zone_tree_t *tree, const knot_dname_t *owner,
 int knot_zone_tree_find_less_or_equal(knot_zone_tree_t *tree,
                                         const knot_dname_t *owner,
                                         const knot_node_t **found,
-                                        const knot_node_t **previous,
-                                        int check_version)
+                                        const knot_node_t **previous)
 {
 	if (tree == NULL || owner == NULL || found == NULL || previous == NULL) {
 		return KNOT_EBADARG;
 	}
 	
 	knot_node_t *f, *p;
-	int ret = knot_zone_tree_get_less_or_equal(tree, owner, &f, &p, check_version);
+	int ret = knot_zone_tree_get_less_or_equal(tree, owner, &f, &p);
 
 	*found = f;
 	*previous = p;
@@ -233,8 +285,7 @@ int knot_zone_tree_find_less_or_equal(knot_zone_tree_t *tree,
 int knot_zone_tree_get_less_or_equal(knot_zone_tree_t *tree,
                                        const knot_dname_t *owner,
                                        knot_node_t **found,
-                                       knot_node_t **previous,
-                                       int check_version)
+                                       knot_node_t **previous)
 {
 	if (tree == NULL || owner == NULL || found == NULL
 	    || previous == NULL) {
@@ -262,37 +313,43 @@ int knot_zone_tree_get_less_or_equal(knot_zone_tree_t *tree,
 	int exact_match = TREE_FIND_LESS_EQUAL(
 	                  tree, knot_zone_tree_node, avl, tmp, &f, &prev);
 
-	knot_node_free(&tmp_data, 0, 0);
+	knot_node_free(&tmp_data, 0);
 	free(tmp);
 
 	*found = (exact_match > 0) ? f->node : NULL;
 
+dbg_zone_exec_detail(
+		char *name = knot_dname_to_str(owner);
+		char *name_f = (f != NULL)
+			? knot_dname_to_str(knot_node_owner(f->node))
+			: "none";
+
+		dbg_zone_detail("Searched for owner %s in zone tree.\n",
+				name);
+		dbg_zone_detail("Exact match: %d\n", exact_match);
+		dbg_zone_detail("Found node: %p: %s.\n", f, name_f);
+		dbg_zone_detail("Previous node: %p.\n", prev);
+
+		free(name);
+		if (f != NULL) {
+			free(name_f);
+		}
+);
+
 	if (exact_match < 0) {
 		// previous is not really previous but should be the leftmost
 		// node in the tree; take it's previous
 		assert(prev != NULL);
-		*previous = knot_node_get_previous(prev->node, check_version);
+		*previous = knot_node_get_previous(prev->node);
 		exact_match = 0;
 	} else if (prev == NULL) {
-		if (!exact_match) {
-			printf("Searched for owner %s in zone tree.\n",
-			       knot_dname_to_str(owner));
-			printf("Exact match: %d\n", exact_match);
-			printf("Found node: %p: %s.\n", f, (f)
-			    ? knot_dname_to_str(knot_node_owner(f->node))
-			    : "none");
-			printf("Previous node: %p: %s.\n", prev, (prev)
-			       ? knot_dname_to_str(knot_node_owner(prev->node))
-			       : "none");
-		}
-
 		// either the returned node is the root of the tree, or
 		// it is the leftmost node in the tree; in both cases
 		// node was found set the previous node of the found
 		// node
 		assert(exact_match > 0);
 		assert(f != NULL);
-		*previous = knot_node_get_previous(f->node, check_version);
+		*previous = knot_node_get_previous(f->node);
 	} else {
 		// otherwise check if the previous node is not an empty
 		// non-terminal
@@ -300,7 +357,7 @@ int knot_zone_tree_get_less_or_equal(knot_zone_tree_t *tree,
 		 *        to an empty non-terminal.
 		 */
 		*previous = (knot_node_rrset_count(prev->node) == 0)
-		            ? knot_node_get_previous(prev->node, check_version)
+		            ? knot_node_get_previous(prev->node)
 		            : prev->node;
 	}
 
@@ -342,7 +399,7 @@ int knot_zone_tree_remove(knot_zone_tree_t *tree,
 	/*! \todo How to know if this was successful? */
 	TREE_REMOVE(tree, knot_zone_tree_node, avl, tmp);
 
-	knot_node_free(&tmp_data, 0, 0);
+	knot_node_free(&tmp_data, 0);
 	free(tmp);
 
 //	*removed = (n) ? n->node : NULL;
@@ -445,24 +502,43 @@ int knot_zone_tree_shallow_copy(knot_zone_tree_t *from,
 
 /*----------------------------------------------------------------------------*/
 
+int knot_zone_tree_deep_copy(knot_zone_tree_t *from,
+                             knot_zone_tree_t *to)
+{
+	if (to == NULL || from == NULL) {
+		return KNOT_EBADARG;
+	}
+	/*
+	 * This function will copy the tree by hand, so that the nodes
+	 * do not have to be inserted the normal way. It should be substantially
+	 * faster.
+	 */
+
+	to->th_cmp = from->th_cmp;
+
+	return knot_zone_tree_deep_copy_node(from->th_root, &to->th_root);
+}
+
+/*----------------------------------------------------------------------------*/
+
 void knot_zone_tree_free(knot_zone_tree_t **tree)
 {
 	if (tree == NULL || *tree == NULL) {
 		return;
 	}
-	knot_zone_tree_free_node((*tree)->th_root, 0, 0);
+	knot_zone_tree_free_node((*tree)->th_root, 0);
 	free(*tree);
 	*tree = NULL;
 }
 
 /*----------------------------------------------------------------------------*/
 
-void knot_zone_tree_deep_free(knot_zone_tree_t **tree, int free_owners)
+void knot_zone_tree_deep_free(knot_zone_tree_t **tree)
 {
 	if (tree == NULL || *tree == NULL) {
 		return;
 	}
-	knot_zone_tree_free_node((*tree)->th_root, 1, free_owners);
+	knot_zone_tree_free_node((*tree)->th_root, 1);
 	free(*tree);
 	*tree = NULL;
 }
