@@ -227,15 +227,21 @@ const knot_zone_t *knot_zonedb_find_zone_for_name(knot_zonedb_t *db,
 	}
 
 	knot_zone_t dummy_zone;
-	dummy_zone.name = (knot_dname_t *)dname;
+	dummy_zone.name = knot_dname_deep_copy(dname);
 	void *found = NULL;
-	int exact_match = gen_tree_find_less_or_equal(db->zone_tree,
-	                                              &dummy_zone,
-	                                              &found);
-	UNUSED(exact_match);
+	
+//	int exact_match = gen_tree_find_less_or_equal(db->zone_tree,
+//	                                              &dummy_zone,
+//	                                              &found);
+//	UNUSED(exact_match);
+	found = gen_tree_find(db->zone_tree, &dummy_zone);
+	while (found == NULL && knot_dname_label_count(dummy_zone.name) > 0) {
+		knot_dname_left_chop_no_copy(dummy_zone.name);
+		found = gen_tree_find(db->zone_tree, &dummy_zone);
+	}
 
 	knot_zone_t *zone = (found) ? (knot_zone_t *)found : NULL;
-
+	
 dbg_zonedb_exec(
 	char *name = knot_dname_to_str(dname);
 	dbg_zonedb("Found zone for name %s: %p\n", name, zone);
@@ -247,6 +253,8 @@ dbg_zonedb_exec(
 		zone = NULL;
 	}
 
+	knot_dname_free(&dummy_zone.name);
+
 	return zone;
 }
 
@@ -286,16 +294,22 @@ knot_zonedb_t *knot_zonedb_copy(const knot_zonedb_t *db)
 	return db_new;
 }
 
+/*----------------------------------------------------------------------------*/
+
 size_t knot_zonedb_zone_count(const knot_zonedb_t *db)
 {
 	return db->zone_count;
 }
 
+/*----------------------------------------------------------------------------*/
+
 struct knot_zone_db_tree_arg {
 	const knot_zone_t **zones;
 	size_t count;
 };
 
+/*----------------------------------------------------------------------------*/
+
 static void save_zone_to_array(void *node, void *data)
 {
 	knot_zone_t *zone = (knot_zone_t *)node;
@@ -305,15 +319,16 @@ static void save_zone_to_array(void *node, void *data)
 	args->zones[args->count++] = zone;
 }
 
+/*----------------------------------------------------------------------------*/
+
 const knot_zone_t **knot_zonedb_zones(const knot_zonedb_t *db)
 {
 	struct knot_zone_db_tree_arg args;
-	args.zones = malloc(sizeof(knot_zone_t) * db->zone_count);
+	args.zones = malloc(sizeof(knot_zone_t *) * db->zone_count);
 	args.count = 0;
 	CHECK_ALLOC_LOG(args.zones, NULL);
 
-	gen_tree_apply_inorder(db->zone_tree, save_zone_to_array,
-	                       &args);
+	gen_tree_apply_inorder(db->zone_tree, save_zone_to_array, &args);
 	assert(db->zone_count == args.count);
 
 	return args.zones;
