@@ -39,7 +39,7 @@ unit_api dname_tests_api = {
 // C will not accept const int in other const definition
 enum { TEST_DOMAINS_OK = 8 };
 
-enum { TEST_DOMAINS_BAD = 4 };
+enum { TEST_DOMAINS_BAD = 5 };
 
 enum { TEST_DOMAINS_NON_FQDN = 6 };
 
@@ -92,7 +92,8 @@ static const struct test_domain
 	{ NULL, "\2ex\3com", 0, "", 0 },
 	{ "ex.com.", NULL, 0, "", 0 },
 	{ "ex.com.\5", "\3ex\3com\0\5", 10, "", 0 },
-	{ "example.com", "\3example\3com", 12, "\x0\x8", 2 }
+	{ "example.com", "\3example\3com", 12, "\x0\x8", 2 },
+	{ "example..", "\7example\0\0", 12, "\x0\x8", 2 }
 };
 
 static int test_dname_create()
@@ -101,7 +102,7 @@ static int test_dname_create()
 	if (dname == NULL
 	    || knot_dname_name(dname) != NULL
 	    || knot_dname_size(dname) != 0
-	    || knot_dname_node(dname, 0) != NULL) {
+	    || knot_dname_node(dname) != NULL) {
 		diag("New domain name not initialized properly!");
 		return 0;
 	}
@@ -161,10 +162,10 @@ static int check_domain_name(const knot_dname_t *dname,
 	}
 
 	if (check_node) {
-		if (knot_dname_node(dname, 0) != NODE_ADDRESS) {
+		if (knot_dname_node(dname) != NODE_ADDRESS) {
 			diag("Node pointer in the created domain name is wrong:"
 			     "%p (should be %p)",
-			     knot_dname_node(dname, 0), NODE_ADDRESS);
+			     knot_dname_node(dname), NODE_ADDRESS);
 			++errors;
 		}
 	}
@@ -194,7 +195,9 @@ static int test_dname_create_from_str_non_fqdn()
 	knot_dname_t *dname = NULL;
 
 	for (int i = 0; i < TEST_DOMAINS_NON_FQDN; ++i) {
-		//note("testing domain: %s", test_domains_non_fqdn[i].str);
+//		note("testing domain: %s, size: %zu",
+//		     test_domains_non_fqdn[i].str,
+//		     strlen(test_domains_non_fqdn[i].str));
 		dname = knot_dname_new_from_str(test_domains_non_fqdn[i].str,
 		          strlen(test_domains_non_fqdn[i].str), NULL);
 		errors += check_domain_name(dname, test_domains_non_fqdn, i, 0);
@@ -490,7 +493,7 @@ static int test_dname_is_subdomain()
 	for (int i = 0; i < TEST_DOMAINS_NON_FQDN; ++i) {
 		dnames_non_fqdn[i] = knot_dname_new_from_str(
 		                test_domains_non_fqdn[i].str,
-		                test_domains_non_fqdn[i].size, NULL);
+		                strlen(test_domains_non_fqdn[i].str), NULL);
 		assert(dnames_non_fqdn[i] != NULL);
 	}
 
@@ -606,6 +609,60 @@ static int test_dname_is_subdomain()
 	return (errors == 0);
 }
 
+static int test_dname_deep_copy() {
+	int errors = 0;
+
+	knot_dname_t *dnames_fqdn[TEST_DOMAINS_OK];
+	knot_dname_t *dnames_non_fqdn[TEST_DOMAINS_NON_FQDN];
+	knot_dname_t *dnames_fqdn_copy[TEST_DOMAINS_OK];
+	knot_dname_t *dnames_non_fqdn_copy[TEST_DOMAINS_NON_FQDN];
+
+	for (int i = 0; i < TEST_DOMAINS_OK; ++i) {
+		dnames_fqdn[i] = knot_dname_new_from_wire(
+		                (const uint8_t *)test_domains_ok[i].wire,
+		                test_domains_ok[i].size, NODE_ADDRESS);
+		assert(dnames_fqdn[i] != NULL);
+	}
+
+	for (int i = 0; i < TEST_DOMAINS_NON_FQDN; ++i) {
+		dnames_non_fqdn[i] = knot_dname_new_from_str(
+		                test_domains_non_fqdn[i].str,
+		                strlen(test_domains_non_fqdn[i].str),
+		                NODE_ADDRESS);
+//		note("Created name: %.*s\n", dnames_non_fqdn[i]->size,
+//		     dnames_non_fqdn[i]->name);
+		assert(dnames_non_fqdn[i] != NULL);
+	}
+
+	/*
+	 * Create copies of the domain names.
+	 */
+	for (int i = 0; i < TEST_DOMAINS_OK; ++i) {
+//		note("Testing %d. FQDN domain.\n", i);
+		dnames_fqdn_copy[i] = knot_dname_deep_copy(dnames_fqdn[i]);
+		assert(dnames_fqdn_copy[i] != NULL);
+		errors += check_domain_name(dnames_fqdn_copy[i],
+		                            test_domains_ok, i, 1);
+		knot_dname_free(&dnames_fqdn_copy[i]);
+		knot_dname_free(&dnames_fqdn[i]);
+	}
+
+	for (int i = 0; i < TEST_DOMAINS_NON_FQDN; ++i) {
+//		note("Testing %d. non-FQDN domain: ", i);
+//		note("%.*s\n", dnames_non_fqdn[i]->size,
+//		     dnames_non_fqdn[i]->name);
+		dnames_non_fqdn_copy[i] =
+		                knot_dname_deep_copy(dnames_non_fqdn[i]);
+		assert(dnames_non_fqdn_copy[i] != NULL);
+		errors += check_domain_name(dnames_non_fqdn_copy[i],
+		                            test_domains_non_fqdn, i, 1);
+		knot_dname_free(&dnames_non_fqdn_copy[i]);
+		knot_dname_free(&dnames_non_fqdn[i]);
+	}
+
+	return (errors == 0);
+}
+
 static int check_wires(const uint8_t *wire1, uint size1,
                          uint8_t *wire2, uint size2)
 {
@@ -652,7 +709,7 @@ static int test_dname_name(knot_dname_t **dnames_fqdn,
 	for (int i = 0; i < TEST_DOMAINS_NON_FQDN; i++) {
 		const uint8_t *tmp_name;
 		tmp_name = knot_dname_name(dnames_non_fqdn[i]);
-		if (!check_wires(tmp_name, dnames_non_fqdn[i]->size - 1,
+		if (!check_wires(tmp_name, dnames_non_fqdn[i]->size,
 			        (uint8_t *)test_domains_non_fqdn[i].wire,
 				test_domains_non_fqdn[i].size)) {
 			diag("Got bad name value from structure: "
@@ -719,7 +776,7 @@ static int test_dname_node(knot_dname_t **dnames_fqdn,
 
 	for (int i = 0; i < TEST_DOMAINS_OK; i++) {
 		const knot_node_t *tmp_node;
-		if ((tmp_node = knot_dname_node(dnames_fqdn[i], 0)) !=
+		if ((tmp_node = knot_dname_node(dnames_fqdn[i])) !=
 		    NODE_ADDRESS) {
 			diag("Got bad node value from structure: "
 			     "%p, should be: %p",
@@ -730,7 +787,7 @@ static int test_dname_node(knot_dname_t **dnames_fqdn,
 
 	for (int i = 0; i < TEST_DOMAINS_NON_FQDN; i++) {
 		const knot_node_t *tmp_node;
-		if ((tmp_node = knot_dname_node(dnames_non_fqdn[i], 0)) !=
+		if ((tmp_node = knot_dname_node(dnames_non_fqdn[i])) !=
 		    NODE_ADDRESS) {
 			diag("Got bad node value from structure: "
 			     "%s, should be: %s",
@@ -757,10 +814,13 @@ static int test_dname_getters(uint type)
 	}
 
 	for (int i = 0; i < TEST_DOMAINS_NON_FQDN; i++) {
-		printf("Creating dname: %s size: %d\n", test_domains_non_fqdn[i].wire, test_domains_non_fqdn[i].size);
+//		note("Creating dname: %s size: %d\n",
+//		     test_domains_non_fqdn[i].wire,
+//		     test_domains_non_fqdn[i].size);
 		dnames_non_fqdn[i] = knot_dname_new_from_str(
 		                test_domains_non_fqdn[i].str,
-		                test_domains_non_fqdn[i].size, NODE_ADDRESS);
+		                strlen(test_domains_non_fqdn[i].str),
+		                NODE_ADDRESS);
 		assert(dnames_non_fqdn[i] != NULL);
 	}
 
@@ -792,7 +852,7 @@ static int test_dname_getters(uint type)
 	return (errors == 0);
 }
 
-static const int KNOT_DNAME_TEST_COUNT = 15;
+static const int KNOT_DNAME_TEST_COUNT = 16;
 
 /*! This helper routine should report number of
  *  scheduled tests for given parameters.
@@ -834,15 +894,11 @@ static int knot_dname_tests_run(int argc, char *argv[])
 	res_final *= res_wire;
 	res_final *= res_str_non_fqdn;
 
-	todo();
 	res = test_dname_getters(0);
 	ok(res, "dname: name");
-	endtodo;
 
-	todo();
 	res = test_dname_getters(1);
 	ok(res, "dname: size");
-	endtodo;
 
 	res = test_dname_getters(2);
 	ok(res, "dname: node");
@@ -871,6 +927,9 @@ static int knot_dname_tests_run(int argc, char *argv[])
 	ok((res = test_dname_is_subdomain()), "dname: is subdomain");
 	res_final *= res;
 
+	ok((res = test_dname_deep_copy()), "dname: deep copy");
+	res_final *= res;
+
 	endskip;  /* create failed */
 
 	return res_final;
