@@ -61,7 +61,7 @@
 /* FIXME: Generate .y and .l to zoneparser/ */
 #include "zparser.h"
 
-enum desclen { PARSER_RRTYPE_DESCRIPTORS_LENGTH = 32770 }; // used to be 101
+enum desclen { PARSER_RRTYPE_DESCRIPTORS_LENGTH = 65536 }; // used to be 101
 
 /* Taken from RFC 1035, section 3.2.4.  */
 static knot_lookup_table_t dns_rrclasses[] = {
@@ -315,7 +315,12 @@ static parser_rrtype_descriptor_t
 	    PARSER_RDATA_WF_BYTE, /* flags */
 	    PARSER_RDATA_WF_SHORT, /* iterations */
 	    PARSER_RDATA_WF_BINARYWITHLENGTH /* salt */ }, true },
-	/* 52 */
+	/* 52 TLSA */
+        { PARSER_RRTYPE_TLSA, T_TLSA, "TLSA", 3,
+          { PARSER_RDATA_WF_BYTE,
+            PARSER_RDATA_WF_BYTE,
+            PARSER_RDATA_WF_BYTE,
+            PARSER_RDATA_WF_BINARY}, true }, 
 
 
     /* In NSD they have indices between 52 and 99 filled with
@@ -364,12 +369,13 @@ static parser_rrtype_descriptor_t
 	/* 32769 */
 	[32769] = { PARSER_RRTYPE_DLV, T_DLV, "DLV", 4,
 	  { PARSER_RDATA_WF_SHORT, PARSER_RDATA_WF_BYTE,
-	    PARSER_RDATA_WF_BYTE, PARSER_RDATA_WF_BINARY } },
+	    PARSER_RDATA_WF_BYTE, PARSER_RDATA_WF_BINARY }}, 
+	[32770 ... 65535] = { PARSER_RRTYPE_TYPEXXX, T_UTYPE, NULL, 1, { PARSER_RDATA_WF_BINARY }}
 };
 
 parser_rrtype_descriptor_t *parser_rrtype_descriptor_by_type(uint16_t type)
 {
-	if (type <= PARSER_RRTYPE_DLV) {
+	if (type <= 65535) {
 		return &knot_rrtype_descriptors[type];
 	}
 	return &knot_rrtype_descriptors[0];
